@@ -5,17 +5,14 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from __future__ import print_function as _
-from __future__ import division as _
-from __future__ import absolute_import as _
 
 from coremltools.converters.mil.mil.passes.pass_registry import register_pass
+from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
 
-
-def commingle_loop_vars_block(block):
+def _commingle_loop_vars_block(block):
     for op in list(block.operations):
         for b in op.blocks:
-            commingle_loop_vars_block(b)
+            _commingle_loop_vars_block(b)
 
         if op.op_type != "while_loop":
             continue
@@ -33,9 +30,8 @@ def commingle_loop_vars_block(block):
           # replace block inputs
           block._block_inputs = op.outputs
 
-
 @register_pass(namespace="nn_backend")
-def commingle_loop_vars(prog):
+class commingle_loop_vars(AbstractGraphPass):
     """
     prog: Program
 
@@ -74,5 +70,6 @@ def commingle_loop_vars(prog):
     # Comment: The resulting program is no longer SSA (multiple assignments on
     # %loop:0).
     """
-    for f_name, f in prog.functions.items():
-        commingle_loop_vars_block(f)
+    def apply(self, prog):
+        for f in prog.functions.values():
+            _commingle_loop_vars_block(f)
