@@ -1,31 +1,34 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2020, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from __future__ import absolute_import as _
-from __future__ import division as _
-from __future__ import print_function as _
-
+from distutils.version import StrictVersion as _StrictVersion
+import gc
 import logging
 import os
-import gc
-
-import six
-
-import tensorflow as tf
-
 from tempfile import mktemp
+import tensorflow as tf
+from tqdm import tqdm as _tqdm
+
 from .basic_graph_ops import fill_outputs
 from .converter import TFConverter
-from .tf_graph_pass import *  # pylint: disable=unused-wildcard-import,wildcard-import
+from .tf_graph_pass import (
+    cond_to_where,
+    constant_propagation,
+    delete_asserts,
+    delete_disconnected_nodes,
+    functionalize_loops,
+    fuse_dilation_conv,
+    insert_get_tuple,
+    quantization_pass,
+    remove_variable_nodes,
+    tensor_array_resource_removal
+)
 from .tfssa import NetworkEnsemble, SSAFunction
 from .parsed_tf_node import ParsedTFNode
 from coremltools.converters._profile_utils import _profile
-from tqdm import tqdm as _tqdm
-from distutils.version import StrictVersion as _StrictVersion
+from coremltools._deps import _get_version
 
 
 class TFLoader:
@@ -105,7 +108,7 @@ class TFLoader:
         logging.debug(msg.format(outputs))
         outputs = outputs if isinstance(outputs, list) else [outputs]
         outputs = [i.split(":")[0] for i in outputs]
-        if tf.__version__ < _StrictVersion("1.13.1"):
+        if _get_version(tf.__version__) < _StrictVersion("1.13.1"):
             return tf.graph_util.extract_sub_graph(graph_def, outputs)
         else:
             return tf.compat.v1.graph_util.extract_sub_graph(graph_def, outputs)
@@ -142,11 +145,11 @@ class TF1Loader(TFLoader):
         elif isinstance(self.model, tf.keras.Model):
             graph_def = self._from_tf_keras_model(self.model)
             return self.extract_sub_graph(graph_def, outputs)
-        elif isinstance(self.model, six.string_types):
+        elif isinstance(self.model, str):
             if not os.path.exists(str(self.model)):
                 raise ValueError('Input model "{}" does not exist'.format(self.model))
             elif os.path.isfile(str(self.model)) and self.model.endswith(".pb"):
-                if tf.__version__ < _StrictVersion("1.13.1"):
+                if _get_version(tf.__version__) < _StrictVersion("1.13.1"):
                     with open(self.model, "rb") as f:
                         gd = tf.GraphDef()
                         gd.ParseFromString(f.read())
@@ -242,7 +245,7 @@ class TF1Loader(TFLoader):
 
         # get model outputs
         output_node_names = []
-        if tf.__version__ < _StrictVersion("1.13.1"):
+        if _get_version(tf.__version__) < _StrictVersion("1.13.1"):
             sess = tf.Session()
         else:
             sess = tf.compat.v1.Session()
@@ -256,7 +259,7 @@ class TF1Loader(TFLoader):
 
         # get frozen graph
         output_graph = mktemp()
-        tf.compat.v1.reset_default_graph() if tf.__version__ >= _StrictVersion("1.13.1") else tf.reset_default_graph()
+        tf.compat.v1.reset_default_graph() if _get_version(tf.__version__) >= _StrictVersion("1.13.1") else tf.reset_default_graph()
         freeze_graph.freeze_graph(
             input_graph=None,
             input_saver=None,
@@ -275,7 +278,7 @@ class TF1Loader(TFLoader):
             saved_model_tags=",".join(saved_model_tags),
         )
 
-        if tf.__version__ < _StrictVersion("1.13.1"):
+        if _get_version(tf.__version__) < _StrictVersion("1.13.1"):
             graph_def = tf.GraphDef()
             with open(output_graph, "rb") as f:
                 graph_def.ParseFromString(f.read())
