@@ -1,22 +1,17 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2020, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from __future__ import print_function as _
-from __future__ import division as _
-from __future__ import absolute_import as _
-
+from coremltools.converters.mil.mil import Builder as mb, types
+from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
 from coremltools.converters.mil.mil.passes.pass_registry import register_pass
-from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil import types
+from coremltools.converters.mil.mil.types.symbolic import is_symbolic
 from coremltools.converters.mil.mil.var import ListVar
 
 
 @register_pass(namespace="tensorflow")
-def backfill_make_list_elem_type(prog):
+class backfill_make_list_elem_type(AbstractGraphPass):
     """
     TF's TensorArrayV3 (represented as make_list in mil) doesn't necessarily
     contain elem shape/type, which is known when write is performed. We
@@ -26,26 +21,26 @@ def backfill_make_list_elem_type(prog):
 
         prog: Program
     """
-    for f_name, f in prog.functions.items():
-        backfill_make_list_elem_type_block(f)
+    def apply(self, prog):
+        for f in prog.functions.values():
+            _backfill_make_list_elem_type_block(f)
 
 
-def backfill_make_list_elem_type_block(block):
+def _backfill_make_list_elem_type_block(block):
     # shallow copy hides changes on f.operations during the loop
-    for op in block.operations[:]:
+    for op in block.operations:
         for b in op.blocks:
-            backfill_make_list_elem_type_block(b)
+            _backfill_make_list_elem_type_block(b)
 
         if op.op_type != "tf_make_list":
             continue
-        # op is `make_list`
 
         if op.outputs[0].elem_type != types.unknown:
             # elem_type of the list is known
             continue
 
         list_var = op.outputs[0]
-        elem_type = infer_elem_type(list_var)  # types.tensor
+        elem_type = _infer_elem_type(list_var)  # types.tensor
         if elem_type is None:
             msg = (
                 "No list_write or list_scatter op to infer make_list "
@@ -56,15 +51,12 @@ def backfill_make_list_elem_type_block(block):
         with block:
             # elem_shape can be runtime-detemrined, which cannot be inferred here at this point,
             # so we add an internal _const_symbolic node to cover both static and dynamic cases.
-            elem_shape_var = mb._const_symbolic(
-                mode="immediate_value",
-                val=elem_type.get_shape(),
-                before_op=op,
-            )
+            elem_shape = [dim.name if is_symbolic(dim) else dim for dim in
+                elem_type.get_shape()]
             new_list = mb.make_list(
                 init_length=op.init_length,
                 dynamic_length=op.dynamic_length,
-                elem_shape=elem_shape_var,
+                elem_shape=tuple(elem_shape),
                 dtype=op.inputs["dtype"],
                 before_op=op,
                 name=op.name,
@@ -76,7 +68,7 @@ def backfill_make_list_elem_type_block(block):
         block.remove_ops([op])
 
 
-def infer_elem_type(list_var):
+def _infer_elem_type(list_var):
     """
     Returns types.tensor. None if failed to infer element type.
     Example:
@@ -110,7 +102,7 @@ def infer_elem_type(list_var):
             block = o.blocks[0]
             # the corresponding Var in body block
             block_var = block.inputs[idx]
-            elem_type = infer_elem_type(block_var)
+            elem_type = _infer_elem_type(block_var)
             if elem_type is not None:
 
                 def _set_types_for_block_inputs(block):
