@@ -5,7 +5,6 @@
 
 import numpy as np
 import os
-import six
 import pytest
 import shutil
 import tempfile
@@ -19,14 +18,14 @@ from coremltools.converters.mil.frontend.tensorflow.converter import TFConverter
 from coremltools.converters.mil.frontend.tensorflow.test.testing_utils import (
     frontend,
     make_tf_graph,
-    run_compare_tf,
     get_tf_keras_io_names,
+    TensorFlowBaseTest
 )
 
 tf = pytest.importorskip("tensorflow")
 
 
-class TestTf1ModelInputsOutputs:
+class TestTf1ModelInputsOutputs(TensorFlowBaseTest):
     def setup(self):
         self.saved_model_dir = tempfile.mkdtemp()
         _, self.model_path_h5 = tempfile.mkstemp(
@@ -52,14 +51,14 @@ class TestTf1ModelInputsOutputs:
             outputs = [outputs]
 
         output_names = [
-            j if isinstance(j, six.string_types) else j.op.name for j in outputs
+            j if isinstance(j, str) else j.op.name for j in outputs
         ]
         mlmodel = converter.convert(model, outputs=output_names)
         assert mlmodel is not None
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
-        run_compare_tf(model, input_dict, outputs)
+        TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
     def test_infer_outputs(self):
         x_shape = (3, 4, 5)
@@ -70,14 +69,14 @@ class TestTf1ModelInputsOutputs:
 
         model, inputs, outputs = build_model
         input_name = (
-            inputs[0] if isinstance(inputs[0], six.string_types) else inputs[0].op.name
+            inputs[0] if isinstance(inputs[0], str) else inputs[0].op.name
         )
         mlmodel = converter.convert(model, inputs=[TensorType(input_name, (3, 4, 5))])
         assert mlmodel is not None
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
-        run_compare_tf(model, input_dict, outputs)
+        TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
     def test_infer_inputs_and_outputs(self):
         x_shape = (3, 4, 5)
@@ -92,7 +91,7 @@ class TestTf1ModelInputsOutputs:
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
-        run_compare_tf(model, input_dict, outputs)
+        TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
     def test_extract_sub_model(self):
         x_shape = (3, 4, 5)
@@ -103,7 +102,7 @@ class TestTf1ModelInputsOutputs:
             return tf.nn.relu(x), tf.math.add(x, y)
 
         model, inputs, outputs = build_model
-        if isinstance(outputs[0], six.string_types):
+        if isinstance(outputs[0], str):
             first_output_name = outputs[0]
         else:
             first_output_name = outputs[0].name.split(":")[0]
@@ -150,8 +149,9 @@ class TestTf1ModelInputsOutputs:
         with pytest.raises(ValueError) as e:
             converter.convert(model, minimum_deployment_target=target)
         e.match(
-            r"Provided minimum deployment target .* version 4 but converted model "
-            r"uses .* available from version 5 onwards.\n    1. Cumsum operation\n"
+            r"Provided minimum deployment target requires model to be of version 4 but converted model "
+            r"uses following features which are available from version 5 onwards. "
+            r"Please use a higher minimum deployment target to convert. \n    1. Cumsum operation\n"
         )
 
     @pytest.mark.parametrize(
@@ -194,10 +194,11 @@ class TestTf1ModelInputsOutputs:
             converter.convert(model, source=frontend)
             e.match(r"Unable to determine the shape of input .*")
 
-        # Test must pass if a user provides shape during conversion,
-        mlmodel = converter.convert(model, source=frontend, inputs=[ct.TensorType(shape=())])
+        mlmodel = converter.convert(model, source=frontend,
+                inputs=[ct.TensorType(shape=(1,))])
         assert mlmodel is not None
 
+    @pytest.mark.xfail(reason="Rank-0 input is not supported", run=True)
     def test_scalar_placeholder_shape(self):
         x_shape = ()  # Scalar Placeholder Shape
 
@@ -211,7 +212,7 @@ class TestTf1ModelInputsOutputs:
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
-        run_compare_tf(model, input_dict, outputs)
+        TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
     def test_shaping_utils(self):
         @make_tf_graph([(None, 4, 5)])
