@@ -1,38 +1,14 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2020, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from __future__ import absolute_import as _
-from __future__ import division as _
-from __future__ import print_function as _
-
+from distutils.version import StrictVersion as _StrictVersion
 import logging as _logging
 import os.path as _os_path
+from tqdm import tqdm as _tqdm
 
 import tensorflow as _tf
-from coremltools.converters.mil.frontend.tensorflow.basic_graph_ops import fill_outputs
-from coremltools.converters.mil.frontend.tensorflow.load import TFLoader
-from coremltools.converters.mil.frontend.tensorflow.parsed_tf_node import ParsedTFNode
-from coremltools.converters.mil.frontend.tensorflow.tf_graph_pass import (
-    constant_propagation,
-    remove_variable_nodes,
-    tensor_array_resource_removal,
-    insert_get_tuple,
-    delete_disconnected_nodes,
-    fuse_dilation_conv,
-)
-from coremltools.converters.mil.frontend.tensorflow.tfssa import (
-    NetworkEnsemble,
-    SSAFunction,
-)
-from coremltools.converters.mil.frontend.tensorflow2.tf_graph_pass import (
-    flatten_sub_graph_namespaces,
-    rewrite_control_flow_functions,
-)
-from six import string_types as _string_types
 from tensorflow.lite.python.util import get_grappler_config as _get_grappler_config
 from tensorflow.lite.python.util import (
     run_graph_optimizations as _run_graph_optimizations,
@@ -45,9 +21,29 @@ from tensorflow.python.framework.function_def_to_graph import (
     function_def_to_graph as _function_def_to_graph,
 )
 from tensorflow.python.keras.saving import saving_utils as _saving_utils
-from tqdm import tqdm as _tqdm
+from tensorflow.python.eager import context
 
 from .converter import TF2Converter
+from coremltools._deps import _get_version
+from coremltools.converters.mil.frontend.tensorflow.basic_graph_ops import fill_outputs
+from coremltools.converters.mil.frontend.tensorflow.load import TFLoader
+from coremltools.converters.mil.frontend.tensorflow.parsed_tf_node import ParsedTFNode
+from coremltools.converters.mil.frontend.tensorflow.tf_graph_pass import (
+    constant_propagation,
+    delete_disconnected_nodes,
+    fuse_dilation_conv,
+    insert_get_tuple,
+    remove_variable_nodes,
+    tensor_array_resource_removal,
+)
+from coremltools.converters.mil.frontend.tensorflow.tfssa import (
+    NetworkEnsemble,
+    SSAFunction,
+)
+from coremltools.converters.mil.frontend.tensorflow2.tf_graph_pass import (
+    flatten_sub_graph_namespaces,
+    rewrite_control_flow_functions,
+)
 
 
 class TF2Loader(TFLoader):
@@ -59,7 +55,7 @@ class TF2Loader(TFLoader):
         ----------
         model: Model created with TensorFlow 2.x
             One of the following model format:
-                - TensorFlow tf.keras.Model object or HDF5 (.h5) file path
+                - TensorFlow tf.keras.Model object or HDF5 (.h5 or .hdf5) file path
                 - TensorFlow SavedModel directory path
                 - TensorFlow list of concrete functions(s)
         debug: bool, optional. Defaults to False.
@@ -81,19 +77,20 @@ class TF2Loader(TFLoader):
         if (
             isinstance(self.model, list)
             or isinstance(self.model, _tf.keras.Model)
-            or isinstance(self.model, _string_types)
+            or isinstance(self.model, str)
         ):
             cfs = []
             if isinstance(self.model, list):
                 cfs = self.model
             if isinstance(self.model, _tf.keras.Model):
                 cfs = self._concrete_fn_from_tf_keras_or_h5(self.model)
-            elif isinstance(self.model, _string_types):
+            elif isinstance(self.model, str):
                 if not _os_path.exists(self.model):
                     raise ValueError(
                         'Input model "{}" does not exist'.format(self.model)
                     )
-                elif _os_path.isfile(self.model) and self.model.endswith(".h5"):
+                elif _os_path.isfile(self.model) \
+                     and (self.model.endswith(".h5") or self.model.endswith(".hdf5")):
                     cfs = self._concrete_fn_from_tf_keras_or_h5(self.model)
                 elif _os_path.isdir(self.model):
                     saved_model = _tf.saved_model.load(self.model)
@@ -217,7 +214,7 @@ class TF2Loader(TFLoader):
 
         for name in sub_graphs:
             sg = graph_fns.get(name)
-            fn_def = sg.definition
+            fn_def = context.get_function_def(name)
             op_input_shapes = sg_input_shapes[name]
             op_input_shapes = op_input_shapes[-len(fn_def.signature.input_arg) :]
             fn_graph = _function_def_to_graph(fn_def, input_shapes=op_input_shapes)
@@ -258,7 +255,7 @@ class TF2Loader(TFLoader):
             graph_dict[fn_name].update({op.name: ParsedTFNode(op.node_def)})
 
         for name, sg in graph._functions.items():
-            sg_def = sg.definition
+            sg_def = context.get_function_def(name)
             if name in sg_input_shapes:
                 input_shapes = sg_input_shapes[name]
                 input_shapes = input_shapes[-len(sg_def.signature.input_arg):]
@@ -298,7 +295,10 @@ class TF2Loader(TFLoader):
         if len(cfs) != 1:
             raise NotImplementedError("Only a single concrete function is supported.")
 
-        frozen_fn = _convert_variables_to_constants_v2(cfs[0], lower_control_flow=False)
+        if _get_version(_tf.__version__) >= _StrictVersion("2.2.0"):
+            frozen_fn = _convert_variables_to_constants_v2(cfs[0], lower_control_flow=False, aggressive_inlining=True)
+        else:
+            frozen_fn = _convert_variables_to_constants_v2(cfs[0], lower_control_flow=False)
         graph_def = frozen_fn.graph.as_graph_def(add_shapes=True)
 
         # run a Grappler's constant folding pass.
