@@ -3,19 +3,34 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
+from distutils.version import StrictVersion as _StrictVersion
+import itertools
+import numpy as np
+import pytest
 import random
+
+
+import coremltools as ct
+from coremltools._deps import _get_version
 from coremltools.converters.mil import testing_reqs
 from coremltools.converters.mil.frontend.tensorflow2.test.testing_utils import (
-    run_compare_tf_keras,
+    TensorFlow2BaseTest
 )
-from coremltools.converters.mil.testing_reqs import *
+from coremltools.converters.mil.frontend.tensorflow.test.testing_utils import (
+    TensorFlowBaseTest
+)
+from coremltools.converters.mil.testing_utils import random_gen
+from ..._utils import is_symbolic_dim_in_prog
 
+TensorFlowBaseTest.run_compare_tf_keras = \
+    TensorFlow2BaseTest.run_compare_tf_keras
 backends = testing_reqs.backends
 
 tf = pytest.importorskip("tensorflow", minversion="2.1.0")
+import tensorflow as _tf  # should be after pytest.importorskip checks
 
 
-class TestActivation:
+class TestActivation(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, op",
         itertools.product(
@@ -35,7 +50,7 @@ class TestActivation:
     def test_layer(self, use_cpu_only, backend, rank, op):
         shape = np.random.randint(low=2, high=4, size=rank)
         model = tf.keras.Sequential([op(batch_input_shape=shape)])
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, -10, 10)],
             use_cpu_only=use_cpu_only,
@@ -75,7 +90,7 @@ class TestActivation:
         model = tf.keras.Sequential(
             [tf.keras.layers.Activation(op, batch_input_shape=shape)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, -10, 10)],
             use_cpu_only=use_cpu_only,
@@ -84,7 +99,7 @@ class TestActivation:
         )
 
 
-class TestBinary:
+class TestBinary(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, op",
         itertools.product(
@@ -106,7 +121,7 @@ class TestBinary:
         input_y = tf.keras.layers.Input(batch_input_shape=tuple(shape))
         out = op()([input_x, input_y])
         model = tf.keras.Model(inputs=[input_x, input_y], outputs=out)
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, -10, 10), random_gen(shape, -10, 10)],
             use_cpu_only=use_cpu_only,
@@ -129,7 +144,7 @@ class TestBinary:
         input_y = tf.keras.layers.Input(batch_input_shape=tuple(shape))
         out = tf.keras.layers.Dot(axes=axes, normalize=normalize)([input_x, input_y])
         model = tf.keras.Model(inputs=[input_x, input_y], outputs=out)
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, -10, 10), random_gen(shape, -10, 10)],
             use_cpu_only=use_cpu_only,
@@ -137,7 +152,7 @@ class TestBinary:
         )
 
 
-class TestConcatenate:
+class TestConcatenate(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, axis",
         itertools.product(
@@ -151,7 +166,7 @@ class TestConcatenate:
             inputs.append(tf.keras.layers.Input(batch_input_shape=tuple(shape)))
         out = tf.keras.layers.Concatenate(axis=axis)(inputs)
         model = tf.keras.Model(inputs=inputs, outputs=out)
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape), random_gen(shape)],
             use_cpu_only=use_cpu_only,
@@ -159,7 +174,7 @@ class TestConcatenate:
         )
 
 
-class TestConvolution:
+class TestConvolution(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         ",".join(
             [
@@ -172,6 +187,7 @@ class TestConvolution:
                 "strides",
                 "dilations",
                 "batch_size",
+                "groups",
             ]
         ),
         itertools.product(
@@ -187,9 +203,10 @@ class TestConvolution:
             [(2, 4, 4, 2, 2, 2), (3, 7, 5, 1, 3, 2)],
             [(1, 1, 1), (1, 2, 3), (1, 3, 2)],
             [
-                (1, 1, 1)
-            ],  # rdar://62951360 (Enhance SpaceToBatchND op to support more dialation rate of Conv)
+                (1, 1, 1), (2, 2, 2),
+            ],
             [1, 3],
+            [1, 2],
         ),
     )
     def test_conv(
@@ -203,17 +220,35 @@ class TestConvolution:
         strides,
         dilations,
         batch_size,
+        groups,
     ):
+        # tensorflow supports groupwise convolution only for version > tf.2.5.0-rc3
+        if _get_version(_tf.__version__) < _StrictVersion("2.5.0") and groups != 1:
+            return
+
+        if op == tf.keras.layers.Conv3D and groups != 1:
+            pytest.xfail("rdar://81629932 (Conv3d with group > 1 tests failing in TF2.0 converter)")
+
+        # TF does not support strides > 1 in conjunction with dilation_rate > 1
+        for i, stride in enumerate(strides):
+            if stride > 1 and dilations[i] > 1:
+                return
+
+        # Dilations with Conv3D not supported yet, since SpaceToBatchND is only supported for ranks 3 or 4
+        for d in dilations:
+            if d > 1 and op == tf.keras.layers.Conv3D:
+                return
+
         s1, s2, s3, k1, k2, k3 = spatial_dim_and_ks
-        c_in, c_out = 2, 3
+        c_in, c_out = 2, 4
         input_shape = None
         kernel_size = None
-        if op in {tf.keras.layers.Conv1D, tf.keras.layers.LocallyConnected1D}:
+        if op == tf.keras.layers.Conv1D:
             input_shape = (batch_size, s3, c_in)
             kernel_size = k3
             strides = strides[2]
             dilations = dilations[2]
-        elif op in {tf.keras.layers.Conv2D, tf.keras.layers.LocallyConnected2D}:
+        elif op == tf.keras.layers.Conv2D:
             input_shape = (batch_size, s2, s3, c_in)
             kernel_size = (k2, k3)
             strides = (strides[1], strides[2])
@@ -222,40 +257,22 @@ class TestConvolution:
             input_shape = (batch_size, s1, s2, s3, c_in)
             kernel_size = (k1, k2, k3)
 
-        if op in {
-            tf.keras.layers.LocallyConnected1D,
-            tf.keras.layers.LocallyConnected2D,
-        }:
-            if padding != "valid":
-                return  # tf.keras only supports "valid"
-            model = tf.keras.Sequential(
-                [
-                    op(
-                        batch_input_shape=input_shape,
-                        filters=c_out,
-                        kernel_size=kernel_size,
-                        strides=strides,
-                        padding=padding.upper(),
-                        data_format=data_format,
-                    )
-                ]
-            )
-        else:
-            model = tf.keras.Sequential(
-                [
-                    op(
-                        batch_input_shape=input_shape,
-                        filters=c_out,
-                        kernel_size=kernel_size,
-                        strides=strides,
-                        padding=padding.upper(),
-                        data_format=data_format,
-                        dilation_rate=dilations,
-                    )
-                ]
-            )
+        model = tf.keras.Sequential(
+            [
+                op(
+                    batch_input_shape=input_shape,
+                    filters=c_out,
+                    kernel_size=kernel_size,
+                    strides=strides,
+                    padding=padding.upper(),
+                    data_format=data_format,
+                    dilation_rate=dilations,
+                    groups=groups,
+                )
+            ]
+        )
 
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(input_shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -278,7 +295,7 @@ class TestConvolution:
         ),
         itertools.product(
             [True, False],
-            ["nn_proto"], # rdar://66998312 ([MIL] concat layer with variable length input support)
+            backends,
             [
                 tf.keras.layers.LocallyConnected1D,
                 tf.keras.layers.LocallyConnected2D,
@@ -288,8 +305,8 @@ class TestConvolution:
             [(2, 4, 4, 2, 2, 2), (3, 7, 5, 1, 3, 2)],
             [(1, 1, 1), (1, 2, 3), (1, 3, 2)],
             [
-                (1, 1, 1)
-            ],  # rdar://62951360 (Enhance SpaceToBatchND op to support more dialation rate of Conv)
+                (1, 1, 1), (2, 2, 2),
+            ],
             [1, 3],
         ),
     )
@@ -356,7 +373,7 @@ class TestConvolution:
                 ]
             )
 
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(input_shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -422,7 +439,7 @@ class TestConvolution:
             ]
         )
 
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(input_shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -443,7 +460,6 @@ class TestConvolution:
             ["same", "valid"],
         ),
     )
-    @pytest.mark.skip(reason="rdar://65198011 (Re-enable Conv3dTranspose and DynamicTile unit tests)")
     def test_conv2d_padding_dynamic_input(
         self,
         use_cpu_only,
@@ -464,7 +480,7 @@ class TestConvolution:
         )(input_layer)
         output_layer = GlobalMaxPooling2D()(layer)
         model = Model(inputs=[input_layer], outputs=[output_layer])
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen((1, 80, 40 ,1), rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -537,15 +553,14 @@ class TestConvolution:
             ]
         )
 
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(input_shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
             backend=backend,
         )
 
-@pytest.mark.skip(reason="rdar://65198011 (Re-enable Conv3dTranspose and DynamicTile unit tests)")
-class TestConvTranspose:
+class TestConvTranspose(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         ",".join(
             [
@@ -616,14 +631,14 @@ class TestConvTranspose:
             ]
         )
 
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(input_shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
             backend=backend,
         )
 
-class TestCropping:
+class TestCropping(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, begin_end",
         itertools.product(
@@ -635,7 +650,7 @@ class TestCropping:
         model = tf.keras.Sequential(
             [tf.keras.layers.Cropping1D(batch_input_shape=shape, cropping=begin_end)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -660,7 +675,7 @@ class TestCropping:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -689,7 +704,7 @@ class TestCropping:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -697,7 +712,7 @@ class TestCropping:
         )
 
 
-class TestDense:
+class TestDense(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, units, activation, use_bias",
         itertools.product(
@@ -721,7 +736,7 @@ class TestDense:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -729,8 +744,7 @@ class TestDense:
         )
 
 
-class TestEmbedding:
-    @pytest.mark.xfail(reason="rdar://63414784")
+class TestEmbedding(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, dims, batch_size, input_length",
         itertools.product(
@@ -755,7 +769,7 @@ class TestEmbedding:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=0, rand_max=dims[0])],
             use_cpu_only=use_cpu_only,
@@ -765,7 +779,7 @@ class TestEmbedding:
         )
 
 
-class TestFlatten:
+class TestFlatten(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, data_format",
         itertools.product(
@@ -780,7 +794,7 @@ class TestFlatten:
         model = tf.keras.Sequential(
             [tf.keras.layers.Flatten(batch_input_shape=shape, data_format=data_format,)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -788,7 +802,7 @@ class TestFlatten:
         )
 
 
-class TestLambda:
+class TestLambda(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, function",
         itertools.product(
@@ -808,14 +822,14 @@ class TestLambda:
         model = tf.keras.Sequential(
             [tf.keras.layers.Lambda(batch_input_shape=shape, function=function,)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-5, rand_max=5)],
             use_cpu_only=use_cpu_only,
             backend=backend,
         )
 
-class TestBatchNormalization:
+class TestBatchNormalization(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, axis, momentum, epsilon",
         itertools.product(
@@ -841,7 +855,7 @@ class TestBatchNormalization:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -869,14 +883,15 @@ class TestBatchNormalization:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
             backend=backend,
         )
 
-class TestInstanceNormalization:
+
+class TestInstanceNormalization(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, axis, epsilon, center, scale",
         itertools.product(
@@ -907,7 +922,7 @@ class TestInstanceNormalization:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -917,23 +932,28 @@ class TestInstanceNormalization:
         )
 
 
-class TestNormalization:
+class TestNormalization(TensorFlowBaseTest):
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rank, axis, epsilon",
+        "use_cpu_only, backend, rank, axis, epsilon, dynamic",
         itertools.product(
-            [True, False], backends, [rank for rank in range(3, 4)], [-1,], [1e-10],
+            [True, False], backends, [rank for rank in range(3, 4)], [-1,], [1e-2, 1e-10], [True, False],
         ),
     )
-    def test_layer_normalization(self, use_cpu_only, backend, rank, axis, epsilon):
+    def test_layer_normalization(self, use_cpu_only, backend, rank, axis, epsilon, dynamic):
         shape = np.random.randint(low=2, high=4, size=rank)
+        keras_shape = shape.tolist()
+
+        if dynamic:
+            keras_shape[0] = None
+
         model = tf.keras.Sequential(
             [
                 tf.keras.layers.LayerNormalization(
-                    batch_input_shape=shape, axis=axis, epsilon=epsilon, trainable=False
+                    batch_input_shape=keras_shape, axis=axis, epsilon=epsilon, trainable=False
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-100, rand_max=100)],
             use_cpu_only=use_cpu_only,
@@ -974,7 +994,7 @@ class TestNormalization:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -984,7 +1004,7 @@ class TestNormalization:
         )
 
 
-class TestPadding:
+class TestPadding(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, op, data_format, padding",
         itertools.product(
@@ -1015,7 +1035,7 @@ class TestPadding:
         model = tf.keras.Sequential(
             [op(batch_input_shape=shape, padding=padding, **kwargs)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1023,7 +1043,7 @@ class TestPadding:
         )
 
 
-class TestPermute:
+class TestPermute(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank_and_perm",
         itertools.product(
@@ -1042,7 +1062,7 @@ class TestPermute:
         model = tf.keras.Sequential(
             [tf.keras.layers.Permute(batch_input_shape=shape, dims=perm)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1050,7 +1070,7 @@ class TestPermute:
         )
 
 
-class TestGlobalPooling:
+class TestGlobalPooling(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, op, data_format",
         itertools.product(
@@ -1087,7 +1107,7 @@ class TestGlobalPooling:
         model = tf.keras.Sequential(
             [op(batch_input_shape=shape, data_format=data_format)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1095,7 +1115,7 @@ class TestGlobalPooling:
         )
 
 
-class TestPooling:
+class TestPooling(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, op, data_format, pool_size",
         itertools.product(
@@ -1128,7 +1148,7 @@ class TestPooling:
         model = tf.keras.Sequential(
             [op(batch_input_shape=shape, pool_size=pool_size, data_format=data_format)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1136,7 +1156,7 @@ class TestPooling:
         )
 
 
-class TestRecurrent:
+class TestRecurrent(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, units, activation, "
         "recurrent_activation, use_bias, return_sequences",
@@ -1175,7 +1195,7 @@ class TestRecurrent:
                 ),
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -1194,7 +1214,7 @@ class TestRecurrent:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -1211,7 +1231,7 @@ class TestRecurrent:
         k_out = tf.keras.layers.TimeDistributed(tf.keras.layers.Dense(1))(lstm)
         model = tf.keras.Model(inputs=k_in, outputs=k_out)
 
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-1, rand_max=1)],
             use_cpu_only=use_cpu_only,
@@ -1221,11 +1241,7 @@ class TestRecurrent:
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends)
     )
-    @pytest.mark.skip(reason="rdar://65198011 (Re-enable unit tests after os update)")
     def test_lstm_dynamic_batch(self, use_cpu_only, backend):
-         # Support dynamic elem_shape <rdar://problem/69522780>
-        if backend != "nn_proto":
-            return
         input_shape = (1, 1280)
         inp = tf.keras.layers.Input(shape=input_shape)
         h0 = tf.keras.layers.Input(shape=(512,))
@@ -1236,7 +1252,7 @@ class TestRecurrent:
                                         recurrent_activation='sigmoid')(inp)
         model = tf.keras.models.Model(inputs=[inp, h0, c0], outputs=[out, hn, cn])
         batch_size = 2
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [
                 random_gen((batch_size, 1, 1280), -1, 1),
@@ -1247,7 +1263,62 @@ class TestRecurrent:
             backend=backend,
         )
 
-class TestRepeatVector:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend", itertools.product([True], backends)
+    )
+    def test_lstm_conversion_static_shapes(self, use_cpu_only, backend):
+        '''
+        Test that intermediate tensor shapes are populated correctly by the converter.
+        That is, there are no symbolic dimensions in the shapes, when conversion is
+        performed with a fixed input shape, irrespective of the shape used in the source model definition.
+        '''
+        def _get_keras_simple_lstm_model(input_shape):
+            input = tf.keras.Input(batch_input_shape=input_shape)
+            output = tf.keras.layers.LSTM(5)(input)
+            keras_model = tf.keras.Model(inputs=input, outputs=output)
+            return keras_model
+
+        def _test_for_symbolic_shapes(keras_input_shape, input_shape_for_conversion, are_symbols_expected):
+            keras_model = _get_keras_simple_lstm_model(keras_input_shape)
+            res = TensorFlowBaseTest.run_compare_tf_keras(
+                        keras_model,
+                        [
+                            random_gen((1, 32, 10), -1, 1),
+                        ],
+                        inputs_for_conversion=[ct.TensorType(shape=input_shape_for_conversion)],
+                        use_cpu_only=use_cpu_only,
+                        backend=backend,
+                    )
+            coreml_model = res[1]
+            mil_prog = coreml_model._get_mil_internal()
+            assert is_symbolic_dim_in_prog(mil_prog) == are_symbols_expected
+
+
+        _test_for_symbolic_shapes(keras_input_shape=(1, 32, 10),
+                                  input_shape_for_conversion=(1, 32, 10),
+                                  are_symbols_expected=False)
+
+        _test_for_symbolic_shapes(keras_input_shape=(None, 32, 10),
+                                  input_shape_for_conversion=(1, 32, 10),
+                                  are_symbols_expected=False)
+
+        _test_for_symbolic_shapes(keras_input_shape=(None, None, 10),
+                                  input_shape_for_conversion=(1, 32, 10),
+                                  are_symbols_expected=False)
+
+        _test_for_symbolic_shapes(keras_input_shape=(None, 32, 10),
+                                  input_shape_for_conversion=(ct.RangeDim(1, 10), 32, 10),
+                                  are_symbols_expected=True)
+
+        if backend[0] != "mlprogram":
+            # FIX ME: model load fails if backend is "mlprogram". rdar://84862138
+            _test_for_symbolic_shapes(keras_input_shape=(None, None, 10),
+                                      input_shape_for_conversion=(ct.RangeDim(1, 10), ct.RangeDim(16, 64), 10),
+                                      are_symbols_expected=True)
+
+
+
+class TestRepeatVector(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, n",
         itertools.product([True, False], backends, [2, 3, 5, 7],),
@@ -1259,7 +1330,7 @@ class TestRepeatVector:
         model = tf.keras.Sequential(
             [tf.keras.layers.RepeatVector(batch_input_shape=shape, n=n)]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1267,7 +1338,7 @@ class TestRepeatVector:
         )
 
 
-class TestReshape:
+class TestReshape(TensorFlowBaseTest):
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, infer_shape",
         itertools.product(
@@ -1287,7 +1358,7 @@ class TestReshape:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1295,7 +1366,7 @@ class TestReshape:
         )
 
 
-class TestSkips:
+class TestSkips(TensorFlowBaseTest):
     # ops in this class should be ignored / pass-through during conversion
 
     @pytest.mark.parametrize(
@@ -1320,7 +1391,7 @@ class TestSkips:
         elif skip_op == tf.keras.layers.SpatialDropout2D:
             shape = shape[:4]
         model = tf.keras.Sequential([skip_op(batch_input_shape=shape, rate=0.5)])
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1338,7 +1409,7 @@ class TestSkips:
                 tf.keras.layers.GaussianNoise(batch_input_shape=shape, stddev=0.5)
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1364,7 +1435,7 @@ class TestSkips:
                 )
             ]
         )
-        run_compare_tf_keras(
+        TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
@@ -1372,9 +1443,9 @@ class TestSkips:
         )
 
 
-class TestUpSampling:
+class TestUpSampling(TensorFlowBaseTest):
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, op, upsample_factor, data_format, interpolation",
+        "use_cpu_only, backend, op, upsample_factor, data_format, interpolation, dynamic",
         itertools.product(
             [True, False],
             backends,
@@ -1386,30 +1457,74 @@ class TestUpSampling:
             [(2, 2, 1), (4, 3, 2), (1, 2, 3)],
             ["channels_first", "channels_last"],
             ["nearest", "bilinear"],
+            [True, False],
         ),
     )
     def test(
-        self, use_cpu_only, backend, op, upsample_factor, data_format, interpolation
+        self, use_cpu_only, backend, op, upsample_factor, data_format, interpolation, dynamic
     ):
         kwargs = {}
         shape = None
+        keras_shape = None
+
         if op == tf.keras.layers.UpSampling1D:
             shape = np.random.randint(low=2, high=4, size=3)
+            keras_shape = np.copy(shape).tolist()
+            if dynamic:
+                keras_shape[1] = None
             upsample_factor = upsample_factor[2]
         elif op == tf.keras.layers.UpSampling2D:
             kwargs = {"data_format": data_format, "interpolation": interpolation}
             shape = np.random.randint(low=2, high=4, size=4)
+            keras_shape = np.copy(shape).tolist()
+            if dynamic:
+                keras_shape[1] = keras_shape[2] = None
             upsample_factor = (upsample_factor[1], upsample_factor[2])
         elif op == tf.keras.layers.UpSampling3D:
             kwargs = {"data_format": data_format}
             shape = np.random.randint(low=2, high=4, size=5)
+            keras_shape = np.copy(shape).tolist()
+            # not support upsampling3D with dynamic input shape, since 6D tensors are produced in that case
+            if dynamic:
+                return
 
         model = tf.keras.Sequential(
-            [op(batch_input_shape=shape, size=upsample_factor, **kwargs)]
+            [op(batch_input_shape=keras_shape, size=upsample_factor, **kwargs)]
         )
-        run_compare_tf_keras(
+        spec = TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
             backend=backend,
+        )[0]
+        # also check if the scale factor are integers
+        if backend[0] == 'neuralnetwork':
+            for layer in spec.neuralNetwork.layers:
+                if layer.WhichOneof('layer') == "upsample":
+                    assert len(layer.upsample.fractionalScalingFactor) == 0
+
+class TestGelu(TensorFlowBaseTest):
+    @pytest.mark.xfail(_get_version(_tf.__version__) < _StrictVersion("2.4.0"), reason="Gelu is a new layer for tf 2.4.0 and above.")
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend, rank, approximate",
+        itertools.product(
+            [True, False],
+            backends,
+            [rank for rank in range(1, 6)],
+            [True, False],
+        ),
+    )
+    def test(
+        self, use_cpu_only, backend, rank, approximate
+    ):
+        shape = np.random.randint(low=2, high=4, size=rank)
+        input = tf.keras.layers.Input(batch_input_shape=tuple(shape))
+        out = tf.keras.activations.gelu(input, approximate=approximate)
+        model = tf.keras.Model(inputs=[input], outputs=out)
+
+        TensorFlowBaseTest.run_compare_tf_keras(
+            model,
+            [random_gen(shape, -10, 10)],
+            use_cpu_only=use_cpu_only,
+            backend=backend,
         )
