@@ -3,26 +3,29 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-
+from collections import OrderedDict
 import logging as _logging
 import torch as _torch
+
 from coremltools._deps import version_lt
-
-from coremltools.converters.mil.input_types import InputType, ImageType
-from coremltools.converters.mil.mil import types
-from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.input_types import ImageType
 from coremltools.converters.mil.mil import (
-    Placeholder,
+    Builder as mb,
     Function,
-    Program,
-    get_new_symbol,
+    types,
+    Program
 )
-from coremltools.converters.mil.mil import Var
 
-from .internal_graph import *
-from .ops import *
+from .internal_graph import InternalTorchIRGraph
+from .ops import convert_nodes
 from .torch_op_registry import _TORCH_OPS_REGISTRY
-from .torchir_passes import *
+from .torchir_passes import (
+    flatten_graph_input_values,
+    flatten_graph_output_values,
+    generate_tensor_assignment_ops,
+    transform_inplace_ops,
+    remove_getattr_nodes
+)
 from .ssa_passes.torch_passes import torch_passes
 
 torch_to_mil_types = {
@@ -145,6 +148,7 @@ class TorchConverter:
         self.output_names = outputs
         self.context = TranscriptionContext()
         raw_graph, params_dict = self._expand_and_optimize_ir(self.torchscript)
+        self.params_dict = params_dict
         self.graph = InternalTorchIRGraph(
             raw_graph, params_dict, self.inputs, cut_at_symbols
         )
@@ -152,6 +156,7 @@ class TorchConverter:
             transform_inplace_ops,
             flatten_graph_input_values,
             flatten_graph_output_values,
+            remove_getattr_nodes,
             generate_tensor_assignment_ops,
         ]
         for p in passes:
@@ -193,6 +198,11 @@ class TorchConverter:
             the set for which no conversion function is registered."""
         return TorchConverter._check_ops(self.graph)
 
+    def convert_const(self):
+        for name, val in self.graph.params.items():
+            const = mb.const(val=val, name=name)
+            self.context.add(const)
+
     def convert(self):
 
         _logging.info("Converting graph.")
@@ -222,9 +232,8 @@ class TorchConverter:
                 self.graph.inputs.keys(), ssa_func_inputs.keys()
             ):
                 self.context.add(ssa_func.inputs[users_name], torch_name=internal_name)
-            for name, val in self.graph.params.items():
-                const = mb.const(val=val, name=name)
-                self.context.add(const)
+
+            self.convert_const()
 
             # Add the rest of the operations
             convert_nodes(self.context, self.graph)
@@ -250,19 +259,144 @@ class TorchConverter:
         self.torch_passes(prog)
         return prog
 
+    def _jit_pass_lower_graph(graph, torchscript):
+        """
+        This graph pass does a similar thing as _torch._C._jit_pass_lower_graph does.
+        It does two things:
+        1. Rename getattr nodes which produce a torch tensor to match the keys in torch model's state_dict
+        2. Construct the params_dict, with the keys similar to state_dict
+
+        To be more specific, this graph pass traces down series of GetAttr ops, and rename the final node to match the torch model state_dict.
+        It also replaces the node inputs by the first created tensor node with the same name.
+
+        Example:
+        Input graph:
+        graph(%self.1 : __torch__.torch.nn.modules.Sequential, %input.1 : Tensor):
+        %2 : prim::GetAttr[name="linear"](%self.1)
+        %3 : prim::GetAttr[name="weight"](%2)
+        %4 : prim::GetAttr[name="bias"](%2)
+        %5 : prim::GetAttr[name="bias"](%2) # duplicated node
+        %6 : conv(%input.1, %3, %4)
+        %7 : add(%input.1, %5)
+        return (%6, %7)
+
+        Output graph:
+        graph(%self.1 : __torch__.torch.nn.modules.Sequential, %input.1 : Tensor):
+        %2 : prim::GetAttr[name="linear"](%self.1)
+        %linear.weight : prim::GetAttr[name="weight"](%2)
+        %linear.bias : prim::GetAttr[name="bias"](%2)
+        %5 : prim::GetAttr[name="bias"](%2) # duplicated node, it is not used now
+        %6 : conv(%input.1, %linear.weight, %linear.bias)
+        %7 : add(%input.1, %linear.bias) # the second input is replaced
+        return (%6, %7)
+
+        And a dictionary {"linear.weight": ..., "linear.bias": ...} is returned, to record the parameters values.
+        Note that, those GetAttr nodes are still in the torch ir graph, but they would be removed in a latter
+        graph pass in the coreml torch internal graph
+
+        """
+
+        """
+        Each getattr node corresponds to a torch object in the torch IR,
+        it could be either:
+        1. torch.nn.modules: submodule in a torch model. For instance, a linear layer in a MLP network.
+        2. torch.Tensor: torch model parameters. For instance, weight for a conv layer.
+        3. torch._C.ScriptObject: quantized torch model parameters.
+        For example, in the graph above, %2 is pointing to the __torch__.torch.nn.modules.Sequential.linear torch submodule.
+        node_to_module_map tracks these mapping.
+
+        node_to_prefic_map track the name for each module,
+        for example, %2 has the prefix name linear and %3 is linear.weight.
+        These names are also keys in the state_dict
+        """
+        node_to_module_map = {}
+        node_to_prefix_map = {}
+        first_node_with_prefix = {}
+        replace_input = {}
+
+        base_module_node = list(graph.inputs())[0]
+        node_to_module_map[base_module_node] = torchscript
+        node_to_prefix_map[base_module_node] = ""
+
+        """
+        params_dict will be contructed in this graph pass. It contains all const tensors needed for the graph computation.
+        And the value is validated against the state_dict if the key is presented in both dictionaries.
+        In some rare cases, state_dict lacks parameters / buffers, so we still need to go through the while graph ourselves.
+        """
+        params_dict = {}
+        state_dict = torchscript.state_dict(keep_vars=True)
+
+        def _check_is_tensor(node, module):
+            if not isinstance(module, _torch.Tensor):
+                return False
+            assert str(node.output().type()) == "Tensor"
+            return True
+
+        def _check_is_quantized_tensor(node, module):
+            if not isinstance(module, _torch._C.ScriptObject):
+                return False
+            # There are three quantized parameters currently supported in Torch:
+            # ref: https://github.com/pytorch/pytorch/blob/master/torch/csrc/jit/passes/lower_graph.cpp
+            supported_quantizes_types = ["LinearPackedParamsBase", "Conv2dPackedParamsBase", "Conv3dPackedParamsBase"]
+            assert node.output().type().name() in supported_quantizes_types
+            return True
+
+        def _get_tensor(module):
+            return module
+
+        def _get_quantized_tensor(module):
+            return tuple(list(module.__getstate__())[:-1])
+
+        def _lower_graph_block(graph):
+            for node in list(graph.nodes()):
+
+                for block in node.blocks():
+                    _lower_graph_block(block)
+
+                for idx, _input in enumerate(list(node.inputs())):
+                    if _input in replace_input:
+                        node.replaceInput(idx, replace_input[_input])
+
+                kind = node.kind().split("::")[1].lower()
+                if kind != "getattr":
+                    continue
+
+                _input = node.input()
+                _output = node.output()
+                attr_name = getattr(node, node.kindOf("name"))("name")
+
+                module = getattr(node_to_module_map[_input], attr_name)
+                node_to_module_map[_output] = module
+
+                input_prefix = node_to_prefix_map[_input]
+                prefix = input_prefix + '.' + attr_name if input_prefix != "" else attr_name
+                node_to_prefix_map[_output] = prefix
+
+                is_tensor = _check_is_tensor(node, module)
+                is_quantized_tensor = _check_is_quantized_tensor(node, module)
+
+                if is_tensor or is_quantized_tensor:
+                    if is_tensor and prefix in state_dict:
+                        assert _torch.equal(module, state_dict[prefix]), "tensor value not consistent between torch ir and state_dict"
+                    if prefix in params_dict:
+                        assert _torch.equal(module, params_dict[prefix])
+                        replace_input[_output] = first_node_with_prefix[prefix]
+                    else:
+                        params_dict[prefix] = _get_tensor(module) if is_tensor else _get_quantized_tensor(module)
+                        first_node_with_prefix[prefix] = _output
+                        _output.setDebugName(prefix)
+
+        _lower_graph_block(graph)
+
+        return graph, params_dict
+
+
     @staticmethod
     def _expand_and_optimize_ir(torchscript):
         """Given a torch.jit.ScriptModule, convert it to a optimized
         torch._C.Graph and dict of model parameter's names to tensors.
         """
-
-        # Recursively replaces all attribute accesses with the sub-graphs of
-        # those modules. The resulting graph will be self-contained and will
-        # not reference into other modules. Params will contain the "trainable"
-        # inputs to the graph.
-        graph, params = _torch._C._jit_pass_lower_graph(
-            torchscript.forward.graph, torchscript._c
-        )
+        graph = torchscript.forward.graph
 
         # From PyTorch code: Inline function and method calls.
         _torch._C._jit_pass_inline(graph)
@@ -315,8 +449,7 @@ class TorchConverter:
         # NOTE: Don't need another DCE, it's included in constant propagation.
         _torch._C._jit_pass_lint(graph)
 
-        input_and_param_names = [val.debugName() for val in graph.inputs()]
-        param_names = input_and_param_names[len(input_and_param_names) - len(params):]
-        params_dict = dict(zip(param_names, params))
+        # Get the params_dict and rename the getattr nodes in the graph
+        graph, params_dict = TorchConverter._jit_pass_lower_graph(graph, torchscript)
 
         return graph, params_dict
