@@ -16,8 +16,7 @@ import torch.nn.functional as F
 from coremltools.converters.mil.mil import types
 from coremltools.converters.mil.mil import Builder as mb
 from coremltools.converters.mil.mil import Function, get_new_symbol
-from coremltools.converters.mil.mil.var import Var
-from coremltools.converters.mil.testing_reqs import random_gen
+from coremltools.converters.mil.testing_utils import random_gen
 
 from .. import ops
 from ..converter import TorchConverter, TranscriptionContext
@@ -240,7 +239,7 @@ class TestTorchOps:
         self._test_elementwise_binary(
             context,
             "Pow",
-            ops.pow_,
+            ops.pow,
             [test_input_1, test_input_2],
             2,
             np.power(test_input_1, test_input_2),
@@ -1272,6 +1271,11 @@ class TestTorchOps:
     ):
         if pad > kernel_size / 2:
             return
+
+        if ceil_mode:
+            if kernel_size == 1 and stride == 2 and pad == 0 and input_shape[-1] == 10:
+                pytest.xfail("Torch ceil_mode does not match exactly with CoreML's ceil_mode. rdar://80050546")
+
         test_input = torch.rand(input_shape)
         expected_result = F.avg_pool1d(
             test_input,
@@ -1306,6 +1310,11 @@ class TestTorchOps:
     ):
         if pad > kernel_size / 2:
             return
+
+        if ceil_mode:
+            if kernel_size == 1 and stride == 2 and pad == 0 and input_shape[-1] == 10:
+                pytest.xfail("Torch ceil_mode does not match exactly with CoreML's ceil_mode. rdar://80050546")
+
         test_input = torch.rand(input_shape)
         expected_result = F.avg_pool2d(
             test_input,
@@ -1337,16 +1346,16 @@ class TestTorchOps:
             [(1, 3, 15), (1, 1, 7), (1, 3, 10)], [1, 3], [1, 2], [0, 1], [False, True]
         ),
     )
-    @pytest.mark.xfail(reason="torch converter for max_pool1d not implemented")
     def test_max_pool1d(
         self, context, input_shape, kernel_size, stride, pad, ceil_mode
     ):
         if pad > kernel_size / 2:
-            # Because this test is xfail, we have to fail rather than
-            # just return here, otherwise these test cases unexpectedly pass.
-            # This can be changed to `return` once the above radar
-            # is fixed and the test is no longer xfail.
-            raise ValueError("pad must be less than half the kernel size")
+            return
+
+        if ceil_mode:
+            if kernel_size == 1 and stride == 2 and pad == 0 and input_shape[-1] == 10:
+                pytest.xfail("Torch ceil_mode does not match exactly with CoreML's ceil_mode. rdar://80050546")
+
         test_input = torch.rand(input_shape)
         expected_result = F.max_pool1d(
             test_input,
@@ -1379,6 +1388,11 @@ class TestTorchOps:
     ):
         if pad > kernel_size / 2:
             return
+
+        if ceil_mode:
+            if kernel_size == 1 and stride == 2 and pad == 0 and input_shape[-1] == 10:
+                pytest.xfail("Torch ceil_mode does not match exactly with CoreML's ceil_mode. rdar://80050546")
+
         test_input = torch.rand(input_shape)
         expected_result = F.max_pool2d(
             test_input,
@@ -1651,11 +1665,9 @@ class TestTorchOps:
         expected_result = torch.tanh(test_input)
         assert np.allclose(expected_result.numpy(), ssa.val)
 
-    # TODO: test for @keepdim==True when the backend bug is fixed.
-    # rdar://62566799
     @pytest.mark.parametrize(
         "input_shape, dim, keepdim",
-        itertools.product([(3, 20, 20), (1, 50, 50)], [0, 1, 2], [False]),
+        itertools.product([(3, 20, 20), (1, 50, 50)], [0, 1, 2], [True, False]),
     )
     def test_argmax(self, context, input_shape, dim, keepdim):
         test_input = torch.rand(*input_shape)
@@ -1691,8 +1703,6 @@ class TestTorchOps:
         expected_result = torch.zeros(size, dtype=ops.NUM_TO_TORCH_DTYPE[dtype])
         np.testing.assert_allclose(expected_result, ssa.val)
 
-    # TODO: Reduce rtol
-    # rdar://62868763 (Numerical discrepancy between torch.exp and coreml MIL exp operation)
     @pytest.mark.parametrize("input_size", [(1, 2, 3, 4), (1,)])
     def test_exp(self, context, input_size):
         test_input = torch.rand(input_size)
