@@ -4,8 +4,10 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import numpy as np
+import pytest
 import torch
 import torch.nn as nn
+
 from coremltools import TensorType, RangeDim
 import coremltools.models.utils as coremltoolsutils
 from ..converter import torch_to_mil_types
@@ -14,14 +16,12 @@ from coremltools._deps import _IS_MACOS, _HAS_TORCH
 from coremltools.converters.mil.mil.types.type_mapping import nptype_from_builtin
 from coremltools.converters.mil.testing_reqs import ct
 from coremltools.converters.mil.testing_utils import ct_convert
-import pytest
 
 
 class ModuleWrapper(nn.Module):
     """
     Helper class to transform torch function into torch nn module.
     This helps to keep the testing interface same for torch functional api.
-
     """
     def __init__(self, function, kwargs=None):
         super(ModuleWrapper, self).__init__()
@@ -51,9 +51,7 @@ def _copy_input_data(input_data):
 
 
 def contains_op(torch, op_string):
-    if hasattr(torch, op_string):
-        return True
-    return False
+    return hasattr(torch, op_string)
 
 
 def convert_to_coreml_inputs(input_description, inputs):
@@ -169,7 +167,7 @@ def convert_and_compare(input_data, model_spec,
     coreml_inputs = convert_to_coreml_inputs(mlmodel.input_description,
                                              input_data)
 
-    if not _IS_MACOS:
+    if not _IS_MACOS or (mlmodel.is_package and coremltoolsutils._macos_version() < (12, 0)):
         return model_spec, mlmodel, coreml_inputs, None
 
     _ , dtype = backend
@@ -214,15 +212,18 @@ class TorchBaseTest(object):
         Traces a model and runs a numerical test.
         Args:
             input_as_shape <bool>: If true generates random input data with shape.
-                expected_results <iterable, optional>: Expected result from running pytorch model.
-
+            expected_results <iterable, optional>: Expected result from running pytorch model.
             converter_input_type: If not None, then pass it to the "inputs" argument to the ct.convert() call.
         """
         model.eval()
         if input_as_shape:
             input_data = generate_input_data(input_data, rand_range)
-        model_spec = torch.jit.script(model) if use_scripting else trace_model(
-            model, _copy_input_data(input_data))
+
+        if use_scripting:
+            model_spec = torch.jit.script(model)
+        else:
+            model_spec = trace_model(model, _copy_input_data(input_data))
+
         model_spec, mlmodel, coreml_inputs, coreml_results = \
             convert_and_compare(
             input_data, model_spec, expected_results=expected_results,
