@@ -5,11 +5,8 @@
 
 import math
 
-import coremltools.converters
-
-from coremltools.converters.mil.mil import get_new_symbol
+from coremltools.converters.mil.mil import get_new_symbol, types
 from coremltools.converters.mil.mil.types.symbolic import is_symbolic
-from ._op_reqs import *
 
 
 def broadcast_shapes(shape_x, shape_y):
@@ -173,7 +170,7 @@ def aggregated_pad(
 
 
 def spatial_dimensions_out_shape(
-    pad_type, input_shape, kernel_shape, strides, dilations=None, custom_pad=None
+    pad_type, input_shape, kernel_shape, strides, dilations=None, custom_pad=None, ceil_mode=False,
 ):
     """
     Args
@@ -194,6 +191,13 @@ def spatial_dimensions_out_shape(
             custom_pad[2*i], custom_pad[2*i+1] are before/after custom padding
             for spatial dim i.
 
+        ceil_mode: determines the padding and output shape.
+             When ceil mode is True:
+                out_dim = floor((in_dim + pad_l + pad_r - kernel_size + (stride-1)) / stride) + 1
+                if (out_dim-1) * stride >= in_dim + pad_l and (pad_l > 0 or pad_r > 0):
+                    out_dim = out_dim - 1
+            When ceil mode is False:
+                out_dim = floor((in_dim + pad_l + pad_r - kernel_size) / stride) + 1
 
     Returns:
         A list of spatial output sizes for each spatial dimension of kernel_shape.
@@ -233,8 +237,68 @@ def spatial_dimensions_out_shape(
         custom_pad=custom_pad,
     )
     effective_ks = effective_kernel(kernel_shape, dilations)
-    out_shape = [
-        (input_shape[r] + pad[r] - effective_ks[r]) // strides[r] + 1
-        for r in range(num_spatial_dims)
-    ]
-    return [dim if not is_symbolic(dim) else get_new_symbol() for dim in out_shape]
+    out_shape = []
+    for r in range(num_spatial_dims):
+        if is_symbolic(input_shape[r]):
+            out_shape.append(get_new_symbol())
+        else:
+            if not ceil_mode:
+                out_shape.append(math.floor((input_shape[r] + pad[r] - effective_ks[r]) / strides[r] + 1))
+            else:
+                out_dim = math.floor((input_shape[r] + pad[r] - effective_ks[r] + strides[r] - 1) / strides[r] + 1)
+                if (out_dim - 1) * strides[r] >= input_shape[r] + pad[r]/2 and pad[r] > 0:
+                    out_dim = out_dim - 1
+                out_shape.append(out_dim)
+    return out_shape
+
+
+def parse_einsum_equation(equation):
+    """
+    Args
+        equation : str
+
+     parse the equation in the following manner:
+     (running example: "nchw,nwhr->nchr")
+
+    step 1: split the equation with delimiter "->"
+        e.g.: this will give "nchw,nwhr" and "nchr"
+
+    step 2: split the LHS equation string with delimiter ","
+        e.g.: this will give input1 : "nchw", input2: "nwhr"
+
+    step 3: map each character to a unique integer, which is incremented.
+            Iterate over input1, input2 and output, in that order.
+            e.g.: input 1, i.e., "nchw" will give vector {0,1,2,3}
+                  input 2, i.e, "nwhr" will produce {0,3,2,4}
+                  output , i.e. "nchr" will produce {0,1,2,4}
+
+    return vectors corresponding to the 2 inputs and the output
+    """
+    input_output_str = equation.split('->')
+    assert len(input_output_str) == 2, "unsupported einsum equation {}".format(equation)
+    input_str = input_output_str[0]
+    output_str = input_output_str[1]
+
+    inputs = input_str.split(',')
+    assert len(inputs) == 2, "unsupported einsum equation {}".format(equation)
+    input1_str = inputs[0]
+    input2_str = inputs[1]
+
+    input1_vec = [-1 for i in range(len(input1_str))]
+    input2_vec = [-1 for i in range(len(input2_str))]
+    output_vec = [-1 for i in range(len(output_str))]
+    map_char_to_int = {}
+
+    def _update_vec(str, vec, map_char_to_int, index):
+        for i, s in enumerate(str):
+            if s not in map_char_to_int:
+                map_char_to_int[s] = index
+                index += 1
+            vec[i] = map_char_to_int[s]
+        return index
+
+    index = _update_vec(input1_str, input1_vec, map_char_to_int, 0)
+    index = _update_vec(input2_str, input2_vec, map_char_to_int, index)
+    index = _update_vec(output_str, output_vec, map_char_to_int, index)
+
+    return input1_vec, input2_vec, output_vec
