@@ -4,6 +4,8 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import math
+import numpy as np
+
 from coremltools.converters.mil.mil.types.symbolic import (
     is_symbolic,
     any_symbolic,
@@ -12,20 +14,40 @@ from coremltools.converters.mil.mil.types.symbolic import (
 from coremltools.converters.mil.mil import (
     get_new_symbol,
     get_new_variadic_symbol,
-    SYMBOL,
-    VALUE,
+    types
+)
+from coremltools.converters.mil.mil.input_type import (
+    BoolInputType,
+    DefaultInputs,
+    FloatInputType,
+    InputSpec,
+    IntInputType,
+    IntOrFloatInputType,
+    IntOrFloatOrBoolInputType,
+    IntTensorInputType,
+    ListOrScalarOrTensorInputType,
+    ScalarOrTensorInputType,
+    StringInputType,
+    TensorInputType,
+    TupleInputType,
+)
+from coremltools.converters.mil.mil.operation import (
     NONE,
+    Operation,
+    precondition,
+    SYMBOL,
+    VALUE
 )
 
-from coremltools.converters.mil.mil import types
-from ._op_reqs import *
+from ._op_reqs import register_op
 from ._utils import promoted_primitive_type
 
 
 @register_op(doc_str="")
 class band_part(Operation):
     """
-    Returns a tensor setting everything outside a center band to zeros for the innermost matrix. Special cases:
+    Returns a tensor setting everything outside a center band to zeros for the innermost
+    matrix. Special cases:
 
     - ``band_part(x, 0, -1)`` returns upper triangular part.
     - ``band_part(x, -1, 0)`` returns lower triangular part.
@@ -33,27 +55,34 @@ class band_part(Operation):
 
     Parameters
     ----------
-    x: tensor<*?, T> (Required)
+    x: tensor<\*?, T> (Required)
         * Input tensor.
     lower: const<i32> (Optional)
-        * Number of lower / below sub-diagonals to keep. If negative, keep entire lower triangle.
-        * Defaults to ``-1`` (keep the entire lower triangle)
+        * Number of lower / below sub-diagonals to keep. If negative, keep entire
+          lower triangle.
+        * Defaults to ``-1`` (keep the entire lower triangle).
     upper: const<i32> (Optional)
-        * Number of upper / above sub-diagonals to keep. If negative, keep entire lower triangle.
-        * Defaults to ``-1`` (keep the entire upper triangle)
+        * Number of upper / above sub-diagonals to keep. If negative, keep entire
+          lower triangle.
+        * Defaults to ``-1`` (keep the entire upper triangle).
 
     Returns
     -------
-    tensor<*?, T>
+    tensor<\*?, T>
         * Same type and shape as the input tensor.
     """
 
     input_spec = InputSpec(
         x=TensorInputType(),
-        lower=IntInputType(const=True, default=-1),
-        upper=IntInputType(const=True, default=-1),
+        lower=IntInputType(const=True, optional=True),
+        upper=IntInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            lower=-1,
+            upper=-1)
+
     def __init__(self, **kwargs):
         super(band_part, self).__init__(**kwargs)
 
@@ -64,28 +93,28 @@ class band_part(Operation):
 @register_op(doc_str="")
 class cumsum(Operation):
     """
-    Returns the cumulative sum the input along the given axis.
+    Returns the cumulative sum of the input along the given axis.
 
     Parameters
     ----------
-    x: tensor<*?, T> (Required)
+    x: tensor<\*?, T> (Required)
         * Input tensor.
     axis: const<i32> (Optional)
         * default to ``0``.
         * Axis for which the cumulative sum is computed.
     exclusive: const<bool> (Optional)
-        * default to ``False``.
+        * Default to ``False``.
         * When set to ``False``, inclusive cumsum is computed, that is the first element of
           the output is identical to the first element in the input.
         * When set to ``True``, exclusive cumsum is computed, which makes the first element
           of output to ``0``.
     reverse: const<bool> (Optional)
-        * default to ``False``.
+        * Default to ``False``.
         * When set to ``True``, perform cumsum in the reverse order.
 
     Returns
     -------
-    tensor<*?, T>
+    tensor<\*?, T>
         * Same type and shape as the input tensor.
 
     Attributes
@@ -95,11 +124,17 @@ class cumsum(Operation):
 
     input_spec = InputSpec(
         x=TensorInputType(),
-        axis=IntInputType(const=True, default=0),
-        exclusive=BoolInputType(const=True, default=False),
-        reverse=BoolInputType(const=True, default=False),
+        axis=IntInputType(const=True, optional=True),
+        exclusive=BoolInputType(const=True, optional=True),
+        reverse=BoolInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            axis=0,
+            exclusive=False,
+            reverse=False)
+
     def __init__(self, **kwargs):
         super(cumsum, self).__init__(**kwargs)
 
@@ -133,7 +168,7 @@ class cumsum(Operation):
 @register_op(doc_str="")
 class fill(Operation):
     """
-    Returns a tensor with given shape filled with a constant value.
+    Returns a tensor with a given shape filled with a constant value.
 
     Parameters
     ----------
@@ -141,12 +176,12 @@ class fill(Operation):
         * Target output tensor shape.
         * ``K`` is the rank of the output tensor. ``shape[k] > 0`` for ``k = 0,..., K-1``.
     value: const<T> (Optional)
-        * default to ``0.0``.
+        * Default to ``0.0``.
         * Constant value to fill in.
 
     Returns
     -------
-    tensor<*?, T>
+    tensor<\*?, T>
         * Tensor with shape determined by the input shape.
 
     Attributes
@@ -156,9 +191,13 @@ class fill(Operation):
 
     input_spec = InputSpec(
         shape=IntTensorInputType(),
-        value=IntOrFloatOrBoolInputType(const=True, default=0.0),
+        value=IntOrFloatOrBoolInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            value=0.)
+
     def __init__(self, **kwargs):
         super(fill, self).__init__(**kwargs)
 
@@ -183,22 +222,29 @@ class fill(Operation):
 class non_maximum_suppression(Operation):
     """
     Applies non-maximum suppression (NMS) on the input box coordinates according
-    to their intersection-over-union (IoU). NMS selects as subset of bounding
-    boxes with the descending scores. Removes boxes that have high
-    intersection-over-union (IOU) overlap with previously selected boxes.
+    to their intersection-over-union (IoU).
+
+    NMS selects a subset of bounding boxes in descending order of score, and removes
+    boxes that have high intersection-over-union (IOU) overlap with previously-selected
+    boxes.
+
 
     Parameters
-    ---------
+    ----------
+
     boxes: tensor<[n, B, 4], T> (Required)
-        * Box coordinates to perform NMS on.
+        * Box coordinates on which to perform NMS.
     scores: tensor<[n, B, K], T> (Required)
-        * Scores for each one of the boxes
+        * Scores for each one of the boxes.
     iou_threshold: const<T> (Required)
-        * The intersection over union (``IoU``) threshold over which boxes are suppressed. NMS remove all overlapping boxes with ``IoU > iou_threshold``.
+        * The intersection over union (``IoU``) threshold over which boxes are
+          suppressed. NMS remove all overlapping boxes with ``IoU > iou_threshold``.
     score_threshold: const<T> (Required)
-        * Before IoU suppression is performed, boxes with class scores below this threshold are rejected.
+        * Before IoU suppression is performed, boxes with class scores below this
+          threshold are rejected.
     max_boxes: const<i32> (Required)
-        * Maximum number of boxes to select. If the number of surviving boxes are less, output is padded up to this number.
+        * Maximum number of boxes to select. If the number of surviving boxes are
+          less, output is padded up to this number.
     per_class_suppression: const<bool> (Optional)
         * Default to ``False``.
         * If ``True``, suppression is performed independently within boxes of each class.
@@ -225,9 +271,13 @@ class non_maximum_suppression(Operation):
         iou_threshold=FloatInputType(const=True),
         score_threshold=FloatInputType(const=True),
         max_boxes=IntInputType(const=True),
-        per_class_suppression=BoolInputType(const=True, default=False),
+        per_class_suppression=BoolInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            per_class_suppression=False)
+
     def __init__(self, **kwargs):
         super(non_maximum_suppression, self).__init__(**kwargs)
 
@@ -252,14 +302,14 @@ class non_zero(Operation):
 
     Parameters
     ----------
-    x: tensor<*?, T> (Required)
+    x: tensor<\*?, T> (Required)
         * Tensor, values selected at indices where its values is not equal to ``0``.
 
     Returns
     -------
     tensor<[N, R], int32>
-        * 2-dimensional tensor contains indices of elements that are non-zero. Each
-          row is the index for a non-zero value.
+        * 2-dimensional tensor contains indices of elements that are non-zero.
+          Each row is the index for a non-zero value.
         * ``N`` is the number of non-zero elements, ``R`` is the rank of the input.
 
     Attributes
@@ -274,7 +324,7 @@ class non_zero(Operation):
 
     def type_inference(self):
         shape = tuple([get_new_symbol(), self.x.rank])
-        return types.tensor(types.int, shape)
+        return types.tensor(types.int32, shape)
 
     @precondition(allow=VALUE)
     def value_inference(self):
@@ -284,13 +334,13 @@ class non_zero(Operation):
 @register_op(doc_str="")
 class one_hot(Operation):
     """
-    Returns one hot vectors whose locations represented in ``indices`` take the ``on_value``,
+    Returns one-hot vectors whose locations represented in ``indices`` take the ``on_value``,
     while other locations take the ``off_value``.
 
     Parameters
     ----------
     indices: tensor<[D],T> (Required)
-        * Tensor, values indicated the locations for each one hot vector to take the ``on_value``.
+        * Tensor, values indicate the locations for each one-hot vector to take the ``on_value``.
     one_got_vector_size: i32 (Required)
         * Indicates the number of returning vectors.
     axis: const i32 (Optional)
@@ -305,8 +355,8 @@ class one_hot(Operation):
 
     Returns
     -------
-    tensor<*?,T>
-        * A tensor contains one hot vectors.
+    tensor<\*?,T>
+        * A tensor that contains one-hot vectors.
 
     Attributes
     ----------
@@ -316,11 +366,18 @@ class one_hot(Operation):
     input_spec = InputSpec(
         indices=IntTensorInputType(),
         one_hot_vector_size=IntInputType(),
-        axis=IntInputType(const=True, default=-1),
-        on_value=IntOrFloatInputType(const=True, default=1),
-        off_value=IntOrFloatInputType(const=True, default=0),
+        axis=IntInputType(const=True, optional=True),
+        on_value=IntOrFloatInputType(const=True, optional=True),
+        off_value=IntOrFloatInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            axis=-1,
+            on_value=1,
+            off_value=0,
+            )
+
     def __init__(self, **kwargs):
         super(one_hot, self).__init__(**kwargs)
 
@@ -342,7 +399,7 @@ class one_hot(Operation):
 
         indices_shape = list(self.indices.shape)
 
-        depth_value = self.one_hot_vector_size.sym_val
+        depth_value = self.one_hot_vector_size.val
         if depth_value is None:
             depth_value = get_new_symbol()
         elif depth_value < 0:
@@ -366,28 +423,31 @@ class pad(Operation):
 
     Parameters
     ----------
-    x: tensor<[*D_in],T>  (Required)
-    * pad: tensor<[2*N],i32> (Required)
-        * ``N <= D_in``: last ``N`` dimensions of ``x`` are padded as follows:
-        * For each dimension ``i`` of ``x`` if ``i >= D_in - N``
+    x: tensor<[\*D_in],T>  (Required)
+
+    pad: tensor<[2\*N],i32> (Required)
+        * ``N <= D_in``: Last ``N`` dimensions of ``x`` are padded as follows: For
+          each dimension ``i`` of ``x`` if ``i >= D_in - N``:
             * pad ``pad[2*i]`` elements before ``x[..,i,..]``
             * pad ``pad[2*i+1]`` elements after ``x[..,i,..]``
-        * If mode is "reflect" then ``pad[2*i]`` and ``pad[2*i+1]`` can be at most ``D[i]-1``.
-        * If mode is "replicate" then ``pad[2*i]`` and ``pad[2*i+1]`` can be at most ``D[i]``.
-    * mode: const<str> (Optional)
-        * Default to 'constant'.
+        * If mode is "reflect" then ``pad[2*i]`` and ``pad[2*i+1]`` can be at
+        most ``D[i]-1``.
+        * If mode is "replicate" then ``pad[2*i]`` and ``pad[2*i+1]`` can be
+        at most ``D[i]``.
+
+    mode: const<str> (Optional)
+        * Default to ``constant``.
         * Must be one of the following values:
-            * constant
-            * reflect
-            * replicate
-    * constant_val: const<T> (Optional)
+          ``constant``, ``reflect``, or ``replicate``.
+
+    constant_val: const<T> (Optional)
         * Default to ``0``.
         * Constant value to pad. Ignored if ``mode != constant``.
 
     Returns
     -------
-    tensor<[*D_out],T>
-        % Tensor with same type as the input.
+    tensor<[\*D_out],T>
+        * Tensor with same type as the input.
 
     Attributes
     ----------
@@ -397,10 +457,16 @@ class pad(Operation):
     input_spec = InputSpec(
         x=TensorInputType(),
         pad=IntTensorInputType(),
-        mode=StringInputType(const=True, default="constant"),
-        constant_val=FloatInputType(const=True, default=0.0),
+        mode=StringInputType(const=True, optional=True),
+        constant_val=FloatInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            mode="constant",
+            constant_val=0.,
+            )
+
     def __init__(self, **kwargs):
         super(pad, self).__init__(**kwargs)
 
@@ -419,8 +485,15 @@ class pad(Operation):
         else:
             pad = pad.val
             pad = pad.copy()
+
+            if len(pad) % 2 != 0:
+                raise ValueError("Number of elements in the argument Pad must be divisible by 2.")
+
             pad = pad.reshape(-1, 2)
 
+            if pad.shape[0] > len(ret_shape):
+                raise ValueError("Number of dimensions specified through pad must less than or equal to rank of input x")
+
             for i in range(len(pad)):
                 ret_shape[-len(pad) + i] = ret_shape[-len(pad) + i] + pad[i][0] + pad[i][1]
 
@@ -451,7 +524,7 @@ class pad(Operation):
 @register_op(doc_str="")
 class range_1d(Operation):
     """
-    Returns a numpy-like 1d range sequence.
+    Returns a numpy-like 1- range sequence.
 
     Parameters
     ----------
@@ -465,7 +538,7 @@ class range_1d(Operation):
     Returns
     -------
     tensor<M, T>
-        * An 1D tensor. where ``M`` is the length of the sequence.
+        * A 1-D tensor, where ``M`` is the length of the sequence.
 
     Attributes
     ----------
@@ -516,19 +589,19 @@ class range_1d(Operation):
 class tile(Operation):
     """
     Returns a new tensor by replicating input ``x`` multiples times.
-    The ``i``th dimention of ``x`` will be replicated ``reps[i]`` times.
+    Dimension ``i`` of ``x`` will be replicated ``reps[i]`` times.
 
     Parameters
     ----------
-    x: tensor<*?, T> (Required)
+    x: tensor<\*?, T> (Required)
         * Input tensor.
     reps: tensor<[rank(x)], int32> (Required)
-        * A 1D tensor with length ``rank(x)`` which indicates number to replicate the input along each dimension.
+        * A 1-D tensor with length ``rank(x)``, which indicates the number to replicate the input along each dimension.
 
     Returns
     -------
-    tensor<*?, T>:
-        * An Nd tensor with same type as the input.
+    tensor<\*?, T>:
+        * An n-D tensor with same type as the input.
 
     Attributes
     ----------
@@ -543,25 +616,32 @@ class tile(Operation):
     def type_inference(self):
         x_type = self.x.dtype
         x_shape = np.array(self.x.shape)
-        reps = self.reps.val
+
+        reps = self.reps.sym_val
+
         if reps is None:
             out_shape = tuple([get_new_symbol() for _ in range(self.x.rank)])
             return types.tensor(x_type, out_shape)
 
-        if len(reps) == 0 or len(reps) > self.x.rank:
+        if len(reps) == 0 or len(reps) != self.x.rank:
             msg = (
                 "Length of the reps ({}) must be at least 1, and "
-                "not greater than the rank of the input x ({})"
+                "equal to the rank of the input x ({})"
             )
             raise ValueError(msg.format(len(reps), self.x.rank))
 
-        if any(i <= 0 for i in reps):
-            raise ValueError("All entries of reps parameter must be greater than 0")
+        out_shape = []
+        for i, rep in enumerate(reps):
+            if not is_symbolic(rep):
+                if rep <= 0:
+                    raise ValueError("All entries of reps parameter must be greater than 0")
 
-        if len(reps) < self.x.rank:
-            reps = [1] * (self.x.rank - len(reps)) + list(reps)
+            if is_symbolic(rep) or is_symbolic(x_shape[i]):
+                out_shape.append(get_new_symbol())
+            else:
+                out_shape.append(rep * x_shape[i])
 
-        out_shape = tuple([reps[i] * x_shape[i] for i in range(len(reps))])
+        out_shape = tuple(out_shape)
 
         return types.tensor(x_type, out_shape)
 
@@ -576,22 +656,23 @@ class tile(Operation):
 @register_op(doc_str="")
 class argsort(Operation):
     """
-    Returns a tensor containing the indices of the sorted values along given axis
+    Returns a tensor containing the indices of the sorted values along a given axis
     of the input tensor.
 
-    Paramters
-    ---------
-    x: <*?, T> (Required)
+    Parameters
+    ----------
+    x: <\*?, T> (Required)
         * Input tensor.
     * axis: const<i32> (Optional)
         * Default to ``-1`` (the last dimension).
         * Axis to perform the operation.
     * ascending: const<bool> (Optional)
-        * True to sort in ascending order. Default to ``False``, sort in descending order.
+        * Default to ``False``, sort in descending order. ``True`` to sort in
+          ascending order.
 
     Returns
     -------
-    tensor<*?, int32>
+    tensor<\*?, int32>
         * Tensor containing the indices of the sorted values
 
     Attributes
@@ -601,10 +682,16 @@ class argsort(Operation):
 
     input_spec = InputSpec(
         x=TensorInputType(),
-        axis=IntInputType(const=True, default=-1),
-        ascending=BoolInputType(const=True, default=False),
+        axis=IntInputType(const=True, optional=True),
+        ascending=BoolInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            axis=-1,
+            ascending=False,
+            )
+
     def __init__(self, **kwargs):
         super(argsort, self).__init__(**kwargs)
 
@@ -613,20 +700,21 @@ class argsort(Operation):
 
     @precondition(allow=VALUE)
     def value_inference(self):
+        # The default np argsort mode is ascending, which is opposite to MIL's argsort op.
         if self.ascending.val:
-            return np.argsort(-self.x.val, axis=self.axis.val)
-        return np.argsort(self.x.val, axis=self.axis.val)
+            return np.argsort(self.x.val, axis=self.axis.val)
+        return np.argsort(-self.x.val, axis=self.axis.val)
 
 
 @register_op(doc_str="")
 class topk(Operation):
     """
-    Returns a tensor containing top or bottom k values and the corresponding
+    Returns a tensor containing top or bottom ``k`` values and the corresponding
     indices of the input tensor along a given axis.
 
     Parameters
     ----------
-    x: <*?, T> (Required)
+    x: <\*?, T> (Required)
         * Input tensor.
     k: const<i32> (Optional)
         * Default to ``1``.
@@ -635,15 +723,15 @@ class topk(Operation):
         * Defaults to ``-1`` (last dimension).
         * Axis to perform the operation.
     * ascending: const<bool> (Optional)
-        * Default to ``False``.
-        * Whether or not to sort in ascending order, sort in descending order.
+        * Default to ``False``, sort in descending order. ``True`` to sort in
+          ascending order.
 
     Returns
     -------
-    tensor<*?, T>
-        * Values of top/bottom ``k`` elements
-    tensor<*?, int32>
-        * Indices of the top/bottom ``k`` elements along axis
+    tensor<\*?, T>
+        * Values of top/bottom ``k`` elements.
+    tensor<\*?, int32>
+        * Indices of the top/bottom ``k`` elements along axis.
 
     Attributes
     ----------
@@ -652,11 +740,18 @@ class topk(Operation):
 
     input_spec = InputSpec(
         x=TensorInputType(),
-        k=IntInputType(const=True, default=1),
-        axis=IntInputType(const=True, default=-1),
-        ascending=BoolInputType(const=True, default=False),
+        k=IntInputType(const=True, optional=True),
+        axis=IntInputType(const=True, optional=True),
+        ascending=BoolInputType(const=True, optional=True),
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            k=1,
+            axis=-1,
+            ascending=False,
+            )
+
     def __init__(self, **kwargs):
         super(topk, self).__init__(**kwargs)
 
@@ -689,7 +784,8 @@ class topk(Operation):
 @register_op(doc_str="")
 class flatten2d(Operation):
     """
-    Flattens input tensor into 2d tensor by flattening dimensions before and after the provided axis
+    Flattens input tensor into 2d tensor by flattening dimensions before and
+    after the provided axis.
 
     Parameters
     ----------
@@ -697,7 +793,7 @@ class flatten2d(Operation):
         * Input tensor.
     * axis: const<f32>  (Optional)
         * Defaults to ``1``.
-        * negative axis is supported.
+        * Negative axis is supported.
 
     Returns
     -------
@@ -719,9 +815,15 @@ class flatten2d(Operation):
     """
 
     input_spec = InputSpec(
-        x=TensorInputType(), axis=IntInputType(const=True, default=1)
+        x=TensorInputType(),
+        axis=IntInputType(const=True, optional=True)
     )
 
+    def default_inputs(self):
+        return DefaultInputs(
+            axis=1,
+            )
+
     def __init__(self, **kwargs):
         super(flatten2d, self).__init__(**kwargs)
 
@@ -746,7 +848,7 @@ class flatten2d(Operation):
 @register_op(doc_str="")
 class shape(Operation):
     """
-    Returns 1-dimensional tensor with shape of input tensor
+    Returns a 1-dimensional tensor with the shape of the input tensor
 
     Parameters
     ----------
@@ -756,7 +858,7 @@ class shape(Operation):
     Returns
     -------
     tensor<K, i32>
-        * Shape of input tensor.
+        * Shape of the input tensor.
         * ``K = x.rank``.
 
     Attributes
@@ -789,35 +891,43 @@ class concat(Operation):
 
     Parameters
     ----------
-    values: Tuple[tensor<[d0, d1, ..., d_axis_i, ..., d_n],T>]  (Required)
-        * The number of dimensions of the input tensors must match, and all dimensions except ``axis`` must be equal.
-        * The tensors may be variadic, but the number of tensors must be determined at compile time (i.e. a tuple).
+    values: Tuple[tensor<[d0, d1, ..., d_axis_i, ..., d_n],T>] (Required)
+        * The number of dimensions of the input tensors must match, and all
+          dimensions except ``axis`` must be equal.
+        * The tensors may be variadic, but the number of tensors must be
+          determined at compile time (i.e. a tuple).
     axis: const<int32> (Required)
-        * The dimension along which to concatenate. Must be in the range ``[-rank(values[i]), rank(values[i]))`` for all ``i``.
+        * The dimension along which to concatenate. Must be in the range
+          ``[-rank(values[i]), rank(values[i]))`` for all ``i``.
     interleave: const<bool> (Optional, Default=False)
-        * If true, concatenate the inputs by interleaving them
+        * If true, concatenate the inputs by interleaving them.
         * If true, all the inputs to this op must have the exact same shape.
-        * e.g.:
-        * in1 : shape (3, 2), value = [[1, 2], [3, 4], [5, 6]]
-        * in2 : shape (3, 2), value = [[7, 8], [9, 10], [11, 12]]
-        * axis = 0
-        *
-        * if interleave = False (default)
-        * output : shape (6, 2)
-        * output[0:3, :] = in1
-        * output[3:6, :] = in2
-        * value = [[1, 2], [3, 4], [5, 6], [7, 8], [9, 10], [11, 12]]
-        *
-        * if interleave = True
-        * output : shape (6, 2)
-        * output[0::2, :] = in1
-        * output[1::2, :] = in2
-        * value = [[1, 2], [7, 8], [3, 4], [9, 10], [5, 6], [11, 12]]
+
+    Examples
+    --------
+
+    .. sourcecode:: python
+
+        in1 : shape (3, 2), value = [[1, 2], [3, 4], [5, 6]]
+        in2 : shape (3, 2), value = [[7, 8], [9, 10], [11, 12]]
+        axis = 0
+
+        if interleave = False (default)
+        output : shape (6, 2)
+        output[0:3, :] = in1
+        output[3:6, :] = in2
+        value = [[1, 2], [3, 4], [5, 6], [7, 8], [9, 10], [11, 12]]
+
+        if interleave = True
+        output : shape (6, 2)
+        output[0::2, :] = in1
+        output[1::2, :] = in2
+        value = [[1, 2], [7, 8], [3, 4], [9, 10], [5, 6], [11, 12]]
 
     Returns
     -------
     tensor<[d0, d1,...d_axis_out, ..., d_n],T>
-        * where ``d_axis_out = sum(d_axis_i)``.
+        * Where ``d_axis_out = sum(d_axis_i)``.
 
     Attributes
     ----------
@@ -826,7 +936,13 @@ class concat(Operation):
 
     input_spec = InputSpec(values=TupleInputType(),
                            axis=IntInputType(const=True),
-                           interleave=BoolInputType(const=True, optional=True, default=False))
+                           interleave=BoolInputType(const=True,
+                                                    optional=True))
+
+    def default_inputs(self):
+        return DefaultInputs(
+            interleave=False,
+            )
 
     def __init__(self, **kwargs):
         super(concat, self).__init__(**kwargs)
@@ -900,24 +1016,30 @@ class concat(Operation):
     @precondition(allow=VALUE | SYMBOL | NONE)
     def value_inference(self):
 
-        is_all_rank_less_than_2 = all([v.rank < 2 for v in self.values])
         values = []
         for v in self.values:
             if v.sym_val is not None:
                 values.append(v.sym_val)
-            else:
-                if v.rank == 1:
-                    values.append(np.array([get_new_symbol() for _ in range(v.shape[0])]))
-                else:
-                    values.append(get_new_symbol())
+                continue
+            if v.rank == 0:
+                values.append(get_new_symbol())
+                continue
+            if any_symbolic(v.shape):
+                values.append(None)
+                continue
 
-        # we only infer value for values whose ranks are all <= 1,
-        # or don't have symbolic values.
-        if any([any_symbolic(v) for v in values]) and not is_all_rank_less_than_2:
-            return None
+            # we support value inference when number of elements for each tensor is less than 10
+            shape = v.shape
+            num_element = np.prod(shape)
+            if num_element > 10:
+                values.append(None)
+                continue
 
-        # skip value inference when interleave on
-        if self.interleave.val:
+            symbolic_tensor = [get_new_symbol() for _ in range(num_element)]
+            symbolic_tensor = np.reshape(np.array(symbolic_tensor), shape)
+            values.append(symbolic_tensor)
+
+        if any([val is None for val in values]):
             return None
 
         if not isinstance(values[0], np.ndarray) or values[0].shape == ():
@@ -933,23 +1055,37 @@ class split(Operation):
 
     Parameters
     ----------
-    x: <*?,T>  (Required)
+    x: <\*?,T>  (Required)
         * The tensor to split.
-        * The tensors may be variadic, but the number of tensors must be determined at compile time (i.e. a tuple).
+        * The tensors may be variadic, but the number of tensors must be determined
+          at compile time (i.e. a tuple).
+
     num_splits: <i32> (Optional)
-        * If specified, divide ``x`` into ``num_splits`` tensors along ``axis``. Its behavior depends on ``split_sizes``:
-            * If ``split_sizes`` is defined, ``num_splits == S``, and the output sizes may be uneven
-            * If ``split_sizes`` is not defined, ``value.shape[axis]`` must be divisible by ``num_splits``, and the output sizes must be even
-        * At least one of ``num_splits`` or ``split_sizes`` must be provided. If ``split_sizes`` length ``S`` cannot be determined at compile time, ``num_splits`` must be supplied to determine the number of outputs.
-    * split_sizes: const<S,i32> (Optional)
-        * Sizes to split to. The sum of ``split_sizes`` must equal to ``value.shape[axis]``.
-    * axis: const<i32> (Required)
-        * The dimension along which to concatenate. Must be in the range ``[-rank(x), rank(x))``.
+        If specified, divide ``x`` into ``num_splits`` tensors along ``axis``.
+        Its behavior depends on ``split_sizes``:
+
+            * If ``split_sizes`` is defined, ``num_splits == S``, and the output
+              sizes may be uneven.
+            * If ``split_sizes`` is not defined, ``value.shape[axis]`` must be
+              divisible by ``num_splits``, and the output sizes must be even.
+
+        At least one of ``num_splits`` or ``split_sizes`` must be provided.
+        If ``split_sizes`` length ``S`` cannot be determined at compile time,
+        ``num_splits`` must be supplied to determine the number of outputs.
+
+    split_sizes: const<S,i32> (Optional)
+        * Sizes to split to. The sum of ``split_sizes`` must equal to
+          ``value.shape[axis]``.
+
+    axis: const<i32> (Required)
+        * The dimension along which to concatenate. Must be in the
+          range ``[-rank(x), rank(x))``.
 
     Returns
     -------
-    Tuple[tensor<*?,T>]
-        * where the length of the tuple is the number of splits (determined from ``num_splits`` or ``split_sizes``).
+    Tuple[tensor<\*?,T>]
+        * Where the length of the tuple is the number of splits (determined
+          from ``num_splits`` or ``split_sizes``).
 
     Attributes
     ----------
@@ -1056,14 +1192,15 @@ class stack(Operation):
     Returns
     -------
     tenor<[d0, d1,...d_axis_out, ..., d_n],T>
-        * where ``d_axis_out = sum(d_axis_i)``
+        * Where ``d_axis_out = sum(d_axis_i)``.
 
     Attributes
     ----------
     T: fp32
     """
 
-    input_spec = InputSpec(values=TupleInputType(), axis=IntInputType(const=True),)
+    input_spec = InputSpec(values=TupleInputType(),
+                           axis=IntInputType(const=True),)
 
     def __init__(self, **kwargs):
         super(stack, self).__init__(**kwargs)
@@ -1105,10 +1242,26 @@ class stack(Operation):
 
         return np.stack(values, self.axis.val)
 
+
 # identity is used for renaming and is rarely necessary. See
 # `loop_invariant_elimination` pass for a rare use case.
 @register_op(doc_str="")
 class identity(Operation):
+    """
+    Returns a tensor with the same shape and contents as input.
+
+    Parameters
+    ----------
+    x: tensor<\*?, T> (Required)
+        * Input tensor.
+
+    Returns
+    -------
+    tensor<\*?, T>
+        * Same type and shape as the input tensor.
+
+    """
+
     input_spec = InputSpec(x=ListOrScalarOrTensorInputType())
 
     def __init__(self, **kwargs):
@@ -1119,4 +1272,4 @@ class identity(Operation):
 
     @precondition(allow=VALUE | SYMBOL)
     def value_inference(self):
-        return self.x.sym_val
\ No newline at end of file
+        return self.x.sym_val
