@@ -4,20 +4,21 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil import get_new_symbol
+from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
 from coremltools.converters.mil.testing_utils import (
     assert_op_count_match,
     assert_model_is_valid,
     get_op_types_in_program,
     apply_pass_and_basic_check,
 )
-import unittest
-import pytest
-
 import numpy as np
+import pytest
+from .reduce_transposes import _find_transpose_compliment
+import unittest
 
 np.random.seed(1984)
 
-
 class TransposeOptimizationPass(unittest.TestCase):
     """"""
 
@@ -1316,6 +1317,139 @@ class TransposeOptimizationPass(unittest.TestCase):
             },
         )
 
+    """
+    Input graph:
+    input(shape=1,4,5,6)--->transpose(axis=[0,3,2,1])--->relu---->split(axis=1, num_splits=2)----->transpose(axis=[0,3,2,1])----->out1(shape=1,4,5,3)
+                                                                        |
+                                                                        v
+                                                                    transpose(axis[0,3,2,1])-------------------------->out2(shape=1,4,5,3)
+
+    Output graph:
+    input(shape=1,4,5,6)------> relu ---->split(axis=3)--->out1(shape=1,4,5,3)
+                                              |
+                                              v
+                                            out2(shape=1,4,5,3)
+    """
+
+    def test_split_nd_pattern_0(self):
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 4, 5, 6))])
+        def prog(x):
+            x1 = mb.transpose(x=x, perm=[0, 3, 2, 1])
+            x1 = mb.relu(x=x1)
+            x2, x3 = mb.split(x=x1, axis=1, num_splits=2)
+            x4 = mb.transpose(x=x2, perm=[0, 3, 2, 1])
+            x5 = mb.transpose(x=x3, perm=[0, 3, 2, 1])
+            return x4, x5
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog),
+            ["transpose", "relu", "split", "transpose", "transpose"],
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["relu", "split"])
+
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 4, 5, 6)},
+            expected_output_shapes={block.outputs[0].name: (1, 4, 5, 3),
+                                    block.outputs[1].name: (1, 4, 5, 3)},
+        )
+
+        self.assertEqual(block.find_ops(op_type="split")[0].axis.val, 3)
+
+    """
+    Input graph:
+    input(shape=1,4,5,6)--->transpose(axis=[0,3,2,1])--->relu---->splitd(axis=1, num_splits=6)----->transpose(axis=[0,3,2,1])----->out1(shape=1,4,5,3)
+                                                                        |
+                                                                        v
+                                                                    transpose(axis[0,3,2,1])-------------------------------------->out2(shape=1,4,5,3)
+
+    Output graph:
+    input(shape=1,4,5,6)------>relu---->split(axis=3)--->out1(shape=1,4,5,3)
+                                              |
+                                              v
+                                            out2(shape=1,4,5,3)
+    """
+
+    def test_split_nd_pattern_1(self):
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 4, 5, 6))])
+        def prog(x):
+            x1 = mb.transpose(x=x, perm=[0, 3, 2, 1])
+            x1 = mb.relu(x=x1)
+            x2, x3, x4, x5, x6, x7 = mb.split(x=x1, axis=1, num_splits=6)
+            x2 = mb.transpose(x=x2, perm=[0, 3, 2, 1])
+            x3 = mb.transpose(x=x3, perm=[0, 3, 2, 1])
+            x4 = mb.transpose(x=x4, perm=[0, 3, 2, 1])
+            x5 = mb.transpose(x=x5, perm=[0, 3, 2, 1])
+            x6 = mb.transpose(x=x6, perm=[0, 3, 2, 1])
+            x7 = mb.transpose(x=x7, perm=[0, 3, 2, 1])
+            return x2, x3, x4, x5, x6, x7
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog),
+            ["transpose", "relu", "split", "transpose", "transpose", "transpose", "transpose", "transpose", "transpose"],
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["relu", "split"])
+
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 4, 5, 6)},
+            expected_output_shapes={block.outputs[0].name: (1, 4, 5, 1),
+                                    block.outputs[1].name: (1, 4, 5, 1),
+                                    block.outputs[2].name: (1, 4, 5, 1),
+                                    block.outputs[3].name: (1, 4, 5, 1),
+                                    block.outputs[4].name: (1, 4, 5, 1),
+                                    block.outputs[5].name: (1, 4, 5, 1)},
+        )
+
+        self.assertEqual(block.find_ops(op_type="split")[0].axis.val, 3)
+
+    """
+    Input graph:
+    input(shape=1,4,5,6)--->transpose(axis=[0,3,2,1])---> split(axis=1, num_splits=2) ----> concat(axis=1) ----->transpose(axis=[0,3,2,1]) ----->out1(shape=1,4,5,6)
+                                                                        |                         ^
+                                                                        v                         | 
+                                                                      relu() ----------------------
+
+    Output graph:
+    input(shape=1,4,5,6)------>split(axis=3)--->concat(axis=3) -------> out1(shape=1,4,5,6)
+                                        |               ^
+                                        v               | 
+                                    relu() --------------
+    """
+
+    def test_split_nd_pattern_2(self):
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 4, 5, 6))])
+        def prog(x):
+            x1 = mb.transpose(x=x, perm=[0, 3, 2, 1])
+            x2, x3 = mb.split(x=x1, axis=1, num_splits=2)
+            x4 = mb.relu(x=x2)
+            x5 = mb.concat(values=[x4, x3], axis=1)
+            x6 = mb.transpose(x=x5, perm=[0, 3, 2, 1])
+            return x6
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog),
+            ["transpose", "split", "relu", "concat", "transpose"]
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["split", "relu", "concat"])
+
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 4, 5, 6)},
+            expected_output_shapes={block.outputs[0].name: (1, 4, 5, 6)},
+        )
+
+        self.assertEqual(block.find_ops(op_type="split")[0].axis.val, 3)
+
     """
     Input graph:
     input(shape=1,5,5,3)----->transpose(axis=[0,3,1,2])
@@ -1660,3 +1794,261 @@ class TransposeOptimizationPass(unittest.TestCase):
             expected_output_shapes={block.outputs[0].name: (1, 2, 5, 5)},
         )
 
+    def test_pass_through_broadcasted_binary_op(self):
+        """
+        Input graph:
+                                                                const (shape=(1,1,1,3))
+                                                                     |
+        input (shape=(1,4,3,2)) --> transpose (shape=(1,2,4,3)) --> add --> transpose --> relu
+
+        Output graph:
+
+                             const (shape=(1,1,3,1))
+                                     |
+        input (shape=(1,4,3,2)) --> add --> relu
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 4, 3, 2))])
+        def prog(x):
+            x = mb.transpose(x=x, perm=[0, 3, 1, 2])
+            x = mb.add(x=x, y=np.array(np.ones(shape=(1, 1, 1, 3))))
+            x = mb.transpose(x=x, perm=[0, 2, 3, 1])
+            x = mb.relu(x=x)
+            return x
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog), ["transpose", "add", "transpose", "relu"],
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["add", "relu"])
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 4, 3, 2)},
+            expected_output_shapes={block.outputs[0].name: (1, 4, 3, 2)},
+        )
+
+    def test_binary_op_with_constant_input(self):
+        """
+        Input graph:
+                                                                const (shape=(4,3))
+                                                                     |
+        input (shape=(1,4,3,2)) --> transpose (shape=(1,2,4,3)) --> add --> transpose --> relu
+
+        Output graph:
+
+                             const (shape=(1,4,3,1))
+                                     |
+        input (shape=(1,4,3,2)) --> add --> relu
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 4, 3, 2))])
+        def prog(x):
+            x = mb.transpose(x=x, perm=[0, 3, 1, 2])
+            x = mb.add(x=x, y=np.array(np.ones(shape=(4, 3))))
+            x = mb.transpose(x=x, perm=[0, 2, 3, 1])
+            x = mb.relu(x=x)
+            return x
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog), ["transpose", "add", "transpose", "relu"],
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["add", "relu"])
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 4, 3, 2)},
+            expected_output_shapes={block.outputs[0].name: (1, 4, 3, 2)},
+        )
+
+    def test_binary_op_with_non_constant_input1(self):
+        """
+        Input graph:
+                                                                input (shape=(3,))
+                                                                     |
+        input (shape=(1,4,3,2)) --> transpose (shape=(1,2,4,3)) --> add --> transpose --> relu
+
+        Output graph:
+
+                                  input (shape=(3,))
+                                     |
+                                 reshape (shape=(1,1,3,1))
+                                     |
+        input (shape=(1,4,3,2)) --> add --> relu
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 4, 3, 2)), mb.TensorSpec(shape=(3,))])
+        def prog(x, y):
+            x = mb.transpose(x=x, perm=[0, 3, 1, 2])
+            x = mb.add(x=x, y=y)
+            x = mb.transpose(x=x, perm=[0, 2, 3, 1])
+            x = mb.relu(x=x)
+            return x
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog), ["transpose", "add", "transpose", "relu"],
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["reshape", "add", "relu"])
+        reshape_op = prog.find_ops(op_type="reshape", exactly_one=True)[0]
+        assert reshape_op.outputs[0].shape == (1, 1, 3, 1)
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 4, 3, 2), "y": (3,)},
+            expected_output_shapes={block.outputs[0].name: (1, 4, 3, 2)},
+        )
+
+    def test_binary_op_with_non_constant_input2(self):
+        """
+        Input graph:
+                                                                input (shape=(3,1,2))
+                                                                     |
+        input (shape=(5,3,4,2)) --> transpose (shape=(4,3,5,2)) --> add --> transpose --> relu
+
+        Output graph:
+
+                                  input (shape=(3,1,2))
+                                     |
+                                 reshape (shape=(1,3,1,2))
+                                     |
+        input (shape=(5,3,4,2)) --> add --> relu
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(5, 3, 4, 2)), mb.TensorSpec(shape=(3, 1, 2))])
+        def prog(x, y):
+            x = mb.transpose(x=x, perm=[2, 1, 0, 3])
+            x = mb.add(x=x, y=y)
+            x = mb.transpose(x=x, perm=[2, 1, 0, 3])
+            x = mb.relu(x=x)
+            return x
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog), ["transpose", "add", "transpose", "relu"],
+        )
+        self.assertEqual(get_op_types_in_program(prog), ["reshape", "add", "relu"])
+        reshape_op = prog.find_ops(op_type="reshape", exactly_one=True)[0]
+        assert reshape_op.outputs[0].shape == (1, 3, 1, 2)
+        assert_model_is_valid(
+            prog,
+            {"x": (5, 3, 4, 2), "y": (3, 1, 2)},
+            expected_output_shapes={block.outputs[0].name: (5, 3, 4, 2)},
+        )
+
+    def test_binary_op_with_non_constant_input3(self):
+        """
+        Input graph:
+                                                                input (shape=(3,1,2))
+                                                                     |
+        input (shape=(s,3,4,2)) --> transpose (shape=(4,3,s,2)) --> add --> transpose --> relu
+
+        Output graph:
+
+                                input (shape=(3,1,2))
+                                     |
+                                 reshape (shape=(1,3,1,2))
+                                     |
+        input (shape=(s,3,4,2)) --> add --> relu
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(get_new_symbol(), 3, 4, 2)), mb.TensorSpec(shape=(3, 1, 2))])
+        def prog(x, y):
+            x = mb.transpose(x=x, perm=[2, 1, 0, 3])
+            x = mb.add(x=x, y=y)
+            x = mb.transpose(x=x, perm=[2, 1, 0, 3])
+            x = mb.relu(x=x)
+            return x
+
+        pass_name = "common::reduce_transposes"
+        PASS_REGISTRY[pass_name](prog)
+        self.assertEqual(get_op_types_in_program(prog), ["reshape", "add", "relu"])
+        reshape_op = prog.find_ops(op_type="reshape", exactly_one=True)[0]
+        assert reshape_op.outputs[0].shape == (1, 3, 1, 2)
+        block = prog.functions["main"]
+        assert_model_is_valid(
+            prog,
+            {"x": (5, 3, 4, 2), "y": (3, 1, 2)},
+            expected_output_shapes={block.outputs[0].name: (5, 3, 4, 2)},
+        )
+
+    def test_binary_op_with_non_constant_input4(self):
+        """
+        Input graph:
+                                                                input (shape=(3,s,2))
+                                                                     |
+        input (shape=(1,3,4,2)) --> transpose (shape=(4,3,1,2)) --> add --> transpose --> relu
+
+        Output graph: same as input graph since the non-transpose input of the add op has symbolic shape
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 3, 4, 2)), mb.TensorSpec(shape=(3, get_new_symbol(), 2))])
+        def prog(x, y):
+            x = mb.transpose(x=x, perm=[2, 1, 0, 3])
+            x = mb.add(x=x, y=y)
+            x = mb.transpose(x=x, perm=[2, 1, 0, 3])
+            x = mb.relu(x=x)
+            return x
+
+        pass_name = "common::reduce_transposes"
+        PASS_REGISTRY[pass_name](prog)
+        self.assertEqual(get_op_types_in_program(prog), ["transpose", "add", "transpose", "relu"])
+        block = prog.functions["main"]
+        assert_model_is_valid(
+            prog,
+            {"x": (1, 3, 4, 2), "y": (3, 10, 2)},
+            expected_output_shapes={block.outputs[0].name: (10, 3, 4, 2)},
+        )
+
+    def test_binary_op_with_non_constant_input5(self):
+        """
+        Input graph:
+                                                                input (shape=(3,4))
+                                                                     |
+        input (shape=(5,3,4,2)) --> transpose (shape=(5,2,3,4)) --> add --> transpose --> relu
+
+        Output graph: same as input graph since transpose compliment for 2nd input of add cannot be represented
+        as a static reshape
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(5, 3, 4, 2)), mb.TensorSpec(shape=(3, 4))])
+        def prog(x, y):
+            x = mb.transpose(x=x, perm=[0, 3, 1, 2])
+            x = mb.add(x=x, y=y)
+            x = mb.transpose(x=x, perm=[0, 2, 3, 1])
+            x = mb.relu(x=x)
+            return x
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "common::reduce_transposes"
+        )
+        self.assertEqual(
+            get_op_types_in_program(prev_prog), ["transpose", "add", "transpose", "relu"],
+        )
+        self.assertEqual(
+            get_op_types_in_program(prog), ["transpose", "add", "transpose", "relu"],
+        )
+        assert_model_is_valid(
+            prog,
+            {"x": (5, 3, 4, 2), "y": (3, 4)},
+            expected_output_shapes={block.outputs[0].name: (5, 3, 4, 2)},
+        )
+
+
+class TestTransposePassUtilityMethods():
+
+    @staticmethod
+    @pytest.mark.parametrize("rank", [1, 2, 3, 4, 5])
+    def test_transpose_compliment_method(rank):
+        x = np.random.rand(*np.random.randint(low=1, high=15, size=rank))
+        perm = np.random.permutation(rank)
+        reverse_perm = _find_transpose_compliment(perm)
+        x_transpose = np.transpose(x, perm)
+        x_transpose_transpose = np.transpose(x_transpose, reverse_perm)
+        np.testing.assert_equal(x, x_transpose_transpose)
