@@ -1,22 +1,17 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2020, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from __future__ import print_function as _
-from __future__ import division as _
-from __future__ import absolute_import as _
 import logging as _logging
 import numpy as _np
 import sympy as _sm
+
 from . import types
 from .block import Function
 from .var import Var
 from .types.symbolic import k_used_symbols, k_num_internal_syms
 from coremltools.converters.mil.input_types import InputType
-import six
 
 
 class Program(object):
@@ -93,8 +88,10 @@ class Placeholder(object):
         """
         if not isinstance(sym_shape, (list, tuple, _np.ndarray)):
             raise ValueError("Illegal shape for Placeholder: {}".format(sym_shape))
+        if len(sym_shape) == 0:
+            raise ValueError('Rank-0 (input {}) is unsupported'.format(name))
         for i, d in enumerate(sym_shape):
-            if not isinstance(d, (_np.generic, six.integer_types, Symbol)):
+            if not isinstance(d, (_np.generic, int, Symbol)):
                 msg = 'Placeholder dim {} in {} is not integer or symbol'
                 raise ValueError(msg.format(i, sym_shape))
         self.sym_shape = sym_shape
@@ -154,6 +151,12 @@ def get_new_symbol(name=None):
     k_num_internal_syms += 1
     return s
 
+def get_existing_symbol(name):
+    global k_used_symbols
+    if name not in k_used_symbols:
+      msg = 'Symbol name {} does not exist'
+      raise ValueError(msg.format(name))
+    return k_used_symbols[name]
 
 class Symbol(_sm.Symbol):
     def __init__(self, sym_name):
@@ -173,5 +176,5 @@ class Symbol(_sm.Symbol):
         if sym_name in k_used_symbols:
             msg = "Symbol `{}` is used already."
             raise ValueError(msg.format(sym_name))
-        k_used_symbols.add(sym_name)
+        k_used_symbols[sym_name] = self
         self.name = sym_name
