@@ -2,21 +2,21 @@
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
-from pathlib import Path
-
-import os
+import copy
 import logging
 import numpy as np
-import copy
+import os
+from pathlib import Path
+import PIL.Image
 import re
 
+import coremltools as ct
+from coremltools._deps import _IS_MACOS
 from coremltools.converters.mil.mil import Program, Function
 from coremltools.converters.mil.mil.passes.quantization_passes import AbstractQuantizationPass
 from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
-from coremltools._deps import _IS_MACOS
+import coremltools.models.utils as coremltoolsutils
 
-import coremltools as ct
-import PIL.Image
 
 np.random.seed(10)
 
@@ -70,7 +70,7 @@ def assert_model_is_valid(
         from coremltools.models.neural_network.printer import print_network_spec
         print_network_spec(mlmodel.get_spec(), style="coding")
 
-    if _IS_MACOS:
+    if _IS_MACOS and (not mlmodel.is_package or coremltoolsutils._macos_version() >= (12, 0)):
         prediction = mlmodel.predict(input_dict, useCPUOnly=True)
         assert prediction is not None
         if expected_output_shapes is not None:
@@ -205,7 +205,7 @@ def compare_backend(
 
         - use_cpu_only: True/False.
     """
-    if _IS_MACOS:
+    if _IS_MACOS and (not mlmodel.is_package or coremltoolsutils._macos_version() >= (12, 0)):
 
         if dtype not in ["fp32", "fp16"]:
             raise ValueError("Unsupported dtype config")
@@ -313,9 +313,6 @@ def get_core_ml_prediction(
     """
     Return predictions of the given model.
     """
-    # Avoid circular import
-    from coremltools.converters.mil.testing_reqs import ct
-
     program = Program()
     with Function(input_placeholders) as ssa_func:
         output_vars = build(**ssa_func.inputs)
@@ -336,7 +333,8 @@ def apply_pass_and_basic_check(prog, pass_name, skip_output_name_check=False):
     Apply pass to the program
     """
     prev_prog = copy.deepcopy(prog)
-    pass_name.apply(prog) if isinstance(pass_name, AbstractQuantizationPass) else PASS_REGISTRY[pass_name](prog)
+    graph_pass = pass_name if isinstance(pass_name, AbstractQuantizationPass) else PASS_REGISTRY[pass_name]
+    graph_pass(prog)
     block = prog.functions["main"]
     prev_block = prev_prog.functions["main"]
     if not skip_output_name_check:
